@@ -41,7 +41,7 @@ import org.apache.kudu.master.Master.TableIdentifierPB;
 @InterfaceStability.Evolving
 public class KuduClient implements AutoCloseable {
 
-  private static final Logger LOG = LoggerFactory.getLogger(AsyncKuduClient.class);
+  private static final Logger LOG = LoggerFactory.getLogger(KuduClient.class);
   public static final long NO_TIMESTAMP = -1;
 
   @InterfaceAudience.LimitedPrivate("Test")
@@ -535,6 +535,18 @@ public class KuduClient implements AutoCloseable {
       return this;
     }
 
+    /**
+     * Sets the default timeout used for connection negotiation.
+     * Optional.
+     * If not provided, defaults to 10s.
+     * @param timeoutMs a timeout in milliseconds
+     * @return this builder
+     */
+    public KuduClientBuilder connectionNegotiationTimeoutMs(long timeoutMs) {
+      clientBuilder.connectionNegotiationTimeoutMs(timeoutMs);
+      return this;
+    }
+
     /**
      * Socket read timeouts are no longer used in the Java client and have no effect.
      * Setting this has no effect.
@@ -542,7 +554,8 @@ public class KuduClient implements AutoCloseable {
      * @return this builder
      * @deprecated socket read timeouts are no longer used
      */
-    @Deprecated public KuduClientBuilder defaultSocketReadTimeoutMs(long timeoutMs) {
+    @Deprecated
+    public KuduClientBuilder defaultSocketReadTimeoutMs(long timeoutMs) {
       LOG.info("defaultSocketReadTimeoutMs is deprecated");
       return this;
     }
@@ -607,6 +620,52 @@ public class KuduClient implements AutoCloseable {
       return this;
     }
 
+    /**
+     * Set the SASL protocol name.
+     * SASL protocol name is used when connecting to a secure (Kerberos-enabled)
+     * cluster. It must match the servers' service principal name (SPN).
+     *
+     * Optional.
+     * If not provided, it will use the default SASL protocol name ("kudu").
+     * @return this builder
+     */
+    public KuduClientBuilder saslProtocolName(String saslProtocolName) {
+      clientBuilder.saslProtocolName(saslProtocolName);
+      return this;
+    }
+
+    /**
+     * Require authentication for the connection to a remote server.
+     *
+     * If it's set to true, the client will require mutual authentication between
+     * the server and the client. If the server doesn't support authentication,
+     * or it's disabled, the client will fail to connect.
+     */
+    public KuduClientBuilder requireAuthentication(boolean requireAuthentication) {
+      clientBuilder.requireAuthentication(requireAuthentication);
+      return this;
+    }
+
+    /**
+     * Require encryption for the connection to a remote server.
+     *
+     * If it's set to REQUIRED or REQUIRED_LOOPBACK, the client will
+     * require encrypting the traffic between the server and the client.
+     * If the server doesn't support encryption, or if it's disabled, the
+     * client will fail to connect.
+     *
+     * Loopback connections are encrypted only if 'encryption_policy' is
+     * set to REQUIRE_LOOPBACK, or if it's required by the server.
+     *
+     * The default value is OPTIONAL, which allows connecting to servers without
+     * encryption as well, but it will still attempt to use it if the server
+     * supports it.
+     */
+    public KuduClientBuilder encryptionPolicy(AsyncKuduClient.EncryptionPolicy encryptionPolicy) {
+      clientBuilder.encryptionPolicy(encryptionPolicy);
+      return this;
+    }
+
     /**
      * Creates a new client that connects to the masters.
      * Doesn't block and won't throw an exception if the masters don't exist.
