@@ -26,6 +26,10 @@ public class CharUtil {
   public static final int MIN_VARCHAR_LENGTH = 1;
   public static final int MAX_VARCHAR_LENGTH = 65535;
 
+  /** Non-constructable utility class. */
+  private CharUtil() {
+  }
+
   /**
    * Convenience method to create column type attributes for VARCHAR columns.
    * @param length the length.
