@@ -17,11 +17,28 @@
 
 package org.apache.kudu.test;
 
+import static org.junit.Assert.assertEquals;
+
+import java.io.IOException;
+import java.math.BigDecimal;
+import java.nio.ByteBuffer;
+import java.sql.Date;
+import java.sql.Timestamp;
+import java.util.ArrayList;
+import java.util.Collections;
+import java.util.List;
+import java.util.concurrent.atomic.AtomicInteger;
+
 import com.google.common.collect.ImmutableList;
 import com.google.common.collect.Iterators;
 import com.google.common.collect.Lists;
 import com.stumbleupon.async.Callback;
 import com.stumbleupon.async.Deferred;
+import org.apache.yetus.audience.InterfaceAudience;
+import org.apache.yetus.audience.InterfaceStability;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
+
 import org.apache.kudu.ColumnSchema;
 import org.apache.kudu.ColumnTypeAttributes;
 import org.apache.kudu.Schema;
@@ -42,23 +59,9 @@ import org.apache.kudu.client.PartialRow;
 import org.apache.kudu.client.RowResult;
 import org.apache.kudu.client.RowResultIterator;
 import org.apache.kudu.client.Upsert;
+import org.apache.kudu.util.CharUtil;
+import org.apache.kudu.util.DateUtil;
 import org.apache.kudu.util.DecimalUtil;
-import org.apache.kudu.util.StringUtil;
-import org.apache.yetus.audience.InterfaceAudience;
-import org.apache.yetus.audience.InterfaceStability;
-import org.slf4j.Logger;
-import org.slf4j.LoggerFactory;
-
-import java.io.IOException;
-import java.math.BigDecimal;
-import java.nio.ByteBuffer;
-import java.sql.Timestamp;
-import java.util.ArrayList;
-import java.util.Collections;
-import java.util.List;
-import java.util.concurrent.atomic.AtomicInteger;
-
-import static org.junit.Assert.assertEquals;
 
 /**
  * Utilities useful for cluster testing.
@@ -94,7 +97,9 @@ public abstract class ClientTestUtil {
     Callback<Object, RowResultIterator> cb = new Callback<Object, RowResultIterator>() {
       @Override
       public Object call(RowResultIterator arg) throws Exception {
-        if (arg == null) return null;
+        if (arg == null) {
+          return null;
+        }
         counter.addAndGet(arg.getNumRows());
         return null;
       }
@@ -130,13 +135,14 @@ public abstract class ClientTestUtil {
    * @param predicates optional predicates to apply to the scan
    * @return the number of rows in the table matching the predicates
    */
-  public static long countRowsInTable(KuduTable table, KuduPredicate... predicates) throws KuduException {
+  public static long countRowsInTable(KuduTable table, KuduPredicate... predicates)
+      throws KuduException {
     KuduScanner.KuduScannerBuilder scanBuilder =
         table.getAsyncClient().syncClient().newScannerBuilder(table);
     for (KuduPredicate predicate : predicates) {
       scanBuilder.addPredicate(predicate);
     }
-    scanBuilder.setProjectedColumnIndexes(ImmutableList.<Integer>of());
+    scanBuilder.setProjectedColumnIndexes(ImmutableList.of());
     return countRowsInScan(scanBuilder.build());
   }
 
@@ -156,7 +162,8 @@ public abstract class ClientTestUtil {
           try (KuduClient contextClient = new KuduClient.KuduClientBuilder(masterAddresses)
                    .defaultAdminOperationTimeoutMs(operationTimeoutMs)
                    .build()) {
-            KuduScanner scanner = KuduScanToken.deserializeIntoScanner(serializedToken, contextClient);
+            KuduScanner scanner =
+                KuduScanToken.deserializeIntoScanner(serializedToken, contextClient);
             try {
               int localCount = 0;
               while (scanner.hasMoreRows()) {
@@ -171,7 +178,7 @@ public abstract class ClientTestUtil {
           }
         }
       });
-      thread.run();
+      thread.start();
       threads.add(thread);
     }
 
@@ -213,7 +220,10 @@ public abstract class ClientTestUtil {
             new ColumnSchema.ColumnSchemaBuilder("null", Type.STRING).nullable(true).build(),
             new ColumnSchema.ColumnSchemaBuilder("timestamp", Type.UNIXTIME_MICROS).build(),
             new ColumnSchema.ColumnSchemaBuilder("decimal", Type.DECIMAL)
-                .typeAttributes(DecimalUtil.typeAttributes(5, 3)).build());
+              .typeAttributes(DecimalUtil.typeAttributes(5, 3)).build(),
+            new ColumnSchema.ColumnSchemaBuilder("varchar", Type.VARCHAR)
+              .typeAttributes(CharUtil.typeAttributes(10)).build(),
+            new ColumnSchema.ColumnSchemaBuilder("date", Type.DATE).build());
 
     return new Schema(columns);
   }
@@ -221,7 +231,7 @@ public abstract class ClientTestUtil {
   public static PartialRow getPartialRowWithAllTypes() {
     Schema schema = getSchemaWithAllTypes();
     // Ensure we aren't missing any types
-    assertEquals(13, schema.getColumnCount());
+    assertEquals(15, schema.getColumnCount());
 
     PartialRow row = schema.newPartialRow();
     row.addByte("int8", (byte) 42);
@@ -229,10 +239,12 @@ public abstract class ClientTestUtil {
     row.addInt("int32", 44);
     row.addLong("int64", 45);
     row.addTimestamp("timestamp", new Timestamp(1234567890));
+    row.addDate("date", DateUtil.epochDaysToSqlDate(0));
     row.addBoolean("bool", true);
     row.addFloat("float", 52.35F);
     row.addDouble("double", 53.35);
     row.addString("string", "fun with ütf\0");
+    row.addVarchar("varchar", "árvíztűrő tükörfúrógép");
     row.addBinary("binary-array", new byte[] { 0, 1, 2, 3, 4 });
     ByteBuffer binaryBuffer = ByteBuffer.wrap(new byte[] { 5, 6, 7, 8, 9 });
     row.addBinary("binary-bytebuffer", binaryBuffer);
@@ -277,17 +289,17 @@ public abstract class ClientTestUtil {
     CreateTableOptions option = new CreateTableOptions();
     option.setRangePartitionColumns(ImmutableList.of("key"));
 
-    PartialRow aLowerBound = schema.newPartialRow();
-    aLowerBound.addInt("key", 0);
-    PartialRow aUpperBound = schema.newPartialRow();
-    aUpperBound.addInt("key", 100);
-    option.addRangePartition(aLowerBound, aUpperBound);
+    PartialRow lowerBoundA = schema.newPartialRow();
+    lowerBoundA.addInt("key", 0);
+    PartialRow upperBoundA = schema.newPartialRow();
+    upperBoundA.addInt("key", 100);
+    option.addRangePartition(lowerBoundA, upperBoundA);
 
-    PartialRow bLowerBound = schema.newPartialRow();
-    bLowerBound.addInt("key", 200);
-    PartialRow bUpperBound = schema.newPartialRow();
-    bUpperBound.addInt("key", 300);
-    option.addRangePartition(bLowerBound, bUpperBound);
+    PartialRow lowerBoundB = schema.newPartialRow();
+    lowerBoundB.addInt("key", 200);
+    PartialRow upperBoundB = schema.newPartialRow();
+    upperBoundB.addInt("key", 300);
+    option.addRangePartition(lowerBoundB, upperBoundB);
 
     PartialRow split = schema.newPartialRow();
     split.addInt("key", 50);
@@ -298,7 +310,8 @@ public abstract class ClientTestUtil {
   /**
    * A generic helper function to create a table with default test options.
    */
-  public static KuduTable createDefaultTable(KuduClient client, String tableName) throws KuduException {
+  public static KuduTable createDefaultTable(KuduClient client, String tableName)
+      throws KuduException {
     return client.createTable(tableName, getBasicSchema(), getBasicCreateTableOptions());
   }
 
@@ -413,8 +426,23 @@ public abstract class ClientTestUtil {
     return table;
   }
 
+  public static Schema createManyVarcharsSchema() {
+    ArrayList<ColumnSchema> columns = new ArrayList<>();
+    columns.add(new ColumnSchema.ColumnSchemaBuilder("key", Type.VARCHAR)
+                  .typeAttributes(CharUtil.typeAttributes(10)).key(true).build());
+    columns.add(new ColumnSchema.ColumnSchemaBuilder("c1", Type.VARCHAR)
+                  .typeAttributes(CharUtil.typeAttributes(10)).build());
+    columns.add(new ColumnSchema.ColumnSchemaBuilder("c2", Type.VARCHAR)
+                  .typeAttributes(CharUtil.typeAttributes(10)).build());
+    columns.add(new ColumnSchema.ColumnSchemaBuilder("c3", Type.VARCHAR)
+                  .typeAttributes(CharUtil.typeAttributes(10)).nullable(true).build());
+    columns.add(new ColumnSchema.ColumnSchemaBuilder("c4", Type.VARCHAR)
+                  .typeAttributes(CharUtil.typeAttributes(10)).nullable(true).build());
+    return new Schema(columns);
+  }
+
   public static Schema createManyStringsSchema() {
-    ArrayList<ColumnSchema> columns = new ArrayList<ColumnSchema>(4);
+    ArrayList<ColumnSchema> columns = new ArrayList<>(4);
     columns.add(new ColumnSchema.ColumnSchemaBuilder("key", Type.STRING).key(true).build());
     columns.add(new ColumnSchema.ColumnSchemaBuilder("c1", Type.STRING).build());
     columns.add(new ColumnSchema.ColumnSchemaBuilder("c2", Type.STRING).build());
@@ -424,7 +452,7 @@ public abstract class ClientTestUtil {
   }
 
   public static Schema createSchemaWithBinaryColumns() {
-    ArrayList<ColumnSchema> columns = new ArrayList<ColumnSchema>();
+    ArrayList<ColumnSchema> columns = new ArrayList<>();
     columns.add(new ColumnSchema.ColumnSchemaBuilder("key", Type.BINARY).key(true).build());
     columns.add(new ColumnSchema.ColumnSchemaBuilder("c1", Type.STRING).build());
     columns.add(new ColumnSchema.ColumnSchemaBuilder("c2", Type.DOUBLE).build());
@@ -433,14 +461,23 @@ public abstract class ClientTestUtil {
   }
 
   public static Schema createSchemaWithTimestampColumns() {
+    ArrayList<ColumnSchema> columns = new ArrayList<>();
+    columns.add(new ColumnSchema.ColumnSchemaBuilder("key", Type.UNIXTIME_MICROS)
+        .key(true).build());
+    columns.add(new ColumnSchema.ColumnSchemaBuilder("c1", Type.UNIXTIME_MICROS)
+        .nullable(true).build());
+    return new Schema(columns);
+  }
+
+  public static Schema createSchemaWithDateColumns() {
     ArrayList<ColumnSchema> columns = new ArrayList<ColumnSchema>();
-    columns.add(new ColumnSchema.ColumnSchemaBuilder("key", Type.UNIXTIME_MICROS).key(true).build());
-    columns.add(new ColumnSchema.ColumnSchemaBuilder("c1", Type.UNIXTIME_MICROS).nullable(true).build());
+    columns.add(new ColumnSchema.ColumnSchemaBuilder("key", Type.DATE).key(true).build());
+    columns.add(new ColumnSchema.ColumnSchemaBuilder("c1", Type.DATE).nullable(true).build());
     return new Schema(columns);
   }
 
   public static Schema createSchemaWithDecimalColumns() {
-    ArrayList<ColumnSchema> columns = new ArrayList<ColumnSchema>();
+    ArrayList<ColumnSchema> columns = new ArrayList<>();
     columns.add(new ColumnSchema.ColumnSchemaBuilder("key", Type.DECIMAL).key(true)
         .typeAttributes(
             new ColumnTypeAttributes.ColumnTypeAttributesBuilder()
