@@ -27,6 +27,7 @@
 #include <ostream>
 #include <random>
 #include <string>
+#include <tuple>
 #include <type_traits>
 #include <unordered_set>
 #include <utility>
@@ -1525,9 +1526,9 @@ class ParameterizedBloomFilterPredicateTest :
     public PredicateTest,
     public ::testing::WithParamInterface<std::tuple<bool, bool>> {};
 
-INSTANTIATE_TEST_CASE_P(, ParameterizedBloomFilterPredicateTest,
-                        ::testing::Combine(::testing::Bool(),
-                                           ::testing::Bool()));
+INSTANTIATE_TEST_SUITE_P(, ParameterizedBloomFilterPredicateTest,
+                         ::testing::Combine(::testing::Bool(),
+                                            ::testing::Bool()));
 
 // Test to verify that an ineffective Bloom filter predicate will be disabled
 // using a pattern of repeated strings.
@@ -1542,12 +1543,13 @@ TEST_P(ParameterizedBloomFilterPredicateTest, TestDisabledBloomFilterWithRepeate
   // Use a set of predetermined strings and populate the table.
   // Create a BF predicate with same set of strings.
   deque<string> values = {"Alice", "Bob", "Charlie", "Doug", "Elizabeth", "Frank", "George",
-                          "Harry"};
+                          "Harry", ""};
 
   // Populate table with a small set of strings that are repeated.
   static constexpr int kNumRows = 10000;
   auto upsert_rows = [&]() {
     int i = 0;
+    int last_flush = i;
     while (i < kNumRows) {
       for (int j = 0; j < values.size() && i < kNumRows; j++, i++) {
         const string &value = values[j];
@@ -1557,8 +1559,9 @@ TEST_P(ParameterizedBloomFilterPredicateTest, TestDisabledBloomFilterWithRepeate
         ASSERT_OK(session->Apply(upsert.release()));
       }
       // TSAN builds timeout and fail on flushing with large number of rows.
-      if ((i % (kNumRows / 10))  == 0) {
+      if (i - last_flush >= 1000) {
         ASSERT_OK(session->Flush());
+        last_flush = i;
       }
     }
     ASSERT_OK(session->Flush());
@@ -1601,10 +1604,10 @@ TEST_P(ParameterizedBloomFilterPredicateTest, TestDisabledBloomFilterWithRepeate
 class ParameterizedPredicateTest : public PredicateTest,
   public ::testing::WithParamInterface<KuduColumnSchema::DataType> {};
 
-INSTANTIATE_TEST_CASE_P(, ParameterizedPredicateTest,
-                        ::testing::Values(KuduColumnSchema::STRING,
-                                          KuduColumnSchema::BINARY,
-                                          KuduColumnSchema::VARCHAR));
+INSTANTIATE_TEST_SUITE_P(, ParameterizedPredicateTest,
+                         ::testing::Values(KuduColumnSchema::STRING,
+                                           KuduColumnSchema::BINARY,
+                                           KuduColumnSchema::VARCHAR));
 
 TEST_P(ParameterizedPredicateTest, TestIndirectDataPredicates) {
   shared_ptr<KuduTable> table = CreateAndOpenTable(GetParam());
