@@ -808,6 +808,20 @@ KuduColumnTypeAttributes KuduColumnSchema::type_attributes() const {
                                   type_attributes.length);
 }
 
+KuduColumnStorageAttributes KuduColumnSchema::storage_attributes() const {
+  ColumnStorageAttributes storage_attributes = DCHECK_NOTNULL(col_)->attributes();
+  KuduColumnStorageAttributes::EncodingType encoding_type;
+  KuduColumnStorageAttributes::StringToEncodingType(
+      kudu::EncodingType_Name(storage_attributes.encoding),
+      &encoding_type);
+  KuduColumnStorageAttributes::CompressionType compression_type;
+  KuduColumnStorageAttributes::StringToCompressionType(
+      kudu::CompressionType_Name(storage_attributes.compression),
+      &compression_type);
+  return KuduColumnStorageAttributes(encoding_type, compression_type,
+                                     storage_attributes.cfile_block_size);
+}
+
 const string& KuduColumnSchema::comment() const {
   return DCHECK_NOTNULL(col_)->comment();
 }
@@ -869,7 +883,7 @@ bool KuduSchema::operator==(const KuduSchema& rhs) const {
 }
 
 bool KuduSchema::operator!=(const KuduSchema& rhs) const {
-  return !(this == &rhs);
+  return !(*this == rhs);
 }
 
 bool KuduSchema::Equals(const KuduSchema& other) const {
