@@ -20,6 +20,7 @@
 #include <algorithm>
 #include <functional>
 #include <memory>
+#include <mutex>
 #include <ostream>
 #include <string>
 
@@ -75,10 +76,30 @@ MonoTime GetRpcDeadline(const KuduClient* c) {
 
 KuduTransaction::Data::Data(const sp::shared_ptr<KuduClient>& client)
     : weak_client_(client),
-      txn_keep_alive_ms_(0) {
+      txn_keep_alive_ms_(0),
+      commit_started_(false) {
   CHECK(client);
 }
 
+MonoDelta KuduTransaction::Data::GetKeepaliveRpcPeriod() const {
+  // Ideally, it would be enough to send a heartbeat message every
+  // txn_keep_alive_ms_ interval, but given scheduling irregularities,
+  // client node timer's precision, and various network delays and latencies,
+  // it's safer to schedule sending keepalive messages from the client side
+  // more frequently.
+  return MonoDelta::FromMilliseconds(
+      std::max<uint32_t>(1, txn_keep_alive_ms_ / 2));
+}
+
+MonoDelta KuduTransaction::Data::GetKeepaliveRpcTimeout() const {
+  // If something goes wrong and keepalive RPC fails, it should be possible
+  // to retry sending keepalive message a couple of times before the transaction
+  // is automatically aborted by the backend after not receiving keepalive
+  // messages for longer than the keepalive timeout for the transaction.
+  return MonoDelta::FromMilliseconds(
+      std::max<uint32_t>(1, txn_keep_alive_ms_ / 4));
+}
+
 Status KuduTransaction::Data::CreateSession(sp::shared_ptr<KuduSession>* session) {
   auto c = weak_client_.lock();
   if (!c) {
@@ -93,6 +114,18 @@ Status KuduTransaction::Data::CreateSession(sp::shared_ptr<KuduSession>* session
   // there isn't much sense duplicating that at the client side.
   sp::shared_ptr<KuduSession> ret(new KuduSession(c, txn_id_));
   ret->data_->Init(ret);
+
+  {
+    std::lock_guard<simple_spinlock> l(commit_started_lock_);
+    if (PREDICT_FALSE(commit_started_)) {
+      return Status::IllegalState("transaction commit has already started");
+    }
+    // Store the information about the newly created session: this is needed
+    // to automatically flush every associated session upon call of
+    // KuduTransaction::Commit() for the corresponding transaction handle.
+    txn_sessions_.emplace_back(ret);
+  }
+
   *session = std::move(ret);
   return Status::OK();
 }
@@ -127,8 +160,6 @@ Status KuduTransaction::Data::Begin(const sp::shared_ptr<KuduTransaction>& txn)
   DCHECK_GT(txn_keep_alive_ms_, 0);
 
   // Start sending regular heartbeats for the new transaction.
-  auto next_run_after = MonoDelta::FromMilliseconds(
-      std::max<uint32_t>(1, txn_keep_alive_ms_ / 2));
   auto m = c->data_->messenger_;
   if (PREDICT_FALSE(!m)) {
     return Status::IllegalState("null messenger in Kudu client");
@@ -139,18 +170,38 @@ Status KuduTransaction::Data::Begin(const sp::shared_ptr<KuduTransaction>& txn)
       [weak_txn](const Status& s) {
         SendTxnKeepAliveTask(s, weak_txn);
       },
-      next_run_after);
+      GetKeepaliveRpcPeriod());
 
   return Status::OK();
 }
 
-Status KuduTransaction::Data::Commit(bool wait) {
+Status KuduTransaction::Data::Commit(CommitMode mode) {
   DCHECK(txn_id_.IsValid());
   auto c = weak_client_.lock();
   if (!c) {
     return Status::IllegalState("associated KuduClient is gone");
   }
 
+  {
+    std::lock_guard<simple_spinlock> l(commit_started_lock_);
+    commit_started_ = true;
+  }
+  // In case of 'synchronous' commit mode (i.e. if waiting for the transaction
+  // to finalize), flush all transactional sessions created out of this handle.
+  // In case of 'asynchronous' commit mode, make sure no transactional session
+  // contains pending operations.
+  for (auto& session : txn_sessions_) {
+    if (mode == CommitMode::WAIT_FOR_COMPLETION) {
+      RETURN_NOT_OK(session->Flush());
+    } else {
+      if (session->HasPendingOperations()) {
+        return Status::IllegalState(
+            "cannot start committing transaction: at least one transactional "
+            "session has write operations pending");
+      }
+    }
+  }
+
   const auto deadline = GetRpcDeadline(c.get());
   CommitTransactionResponsePB resp;
   {
@@ -168,7 +219,7 @@ Status KuduTransaction::Data::Commit(bool wait) {
   if (resp.has_error()) {
     return StatusFromPB(resp.error().status());
   }
-  if (wait) {
+  if (mode == CommitMode::WAIT_FOR_COMPLETION) {
     RETURN_NOT_OK(WaitForTxnCommitToFinalize(c.get(), deadline, txn_id_));
   }
   return Status::OK();
@@ -261,13 +312,11 @@ Status KuduTransaction::Data::Deserialize(
     auto m = client->data_->messenger_;
     if (PREDICT_TRUE(m)) {
       sp::weak_ptr<KuduTransaction> weak_txn(ret);
-      auto next_run_after = MonoDelta::FromMilliseconds(
-          std::max<uint32_t>(1, ret->data_->txn_keep_alive_ms_ / 2));
       m->ScheduleOnReactor(
           [weak_txn](const Status& s) {
             SendTxnKeepAliveTask(s, weak_txn);
           },
-          next_run_after);
+          ret->data_->GetKeepaliveRpcPeriod());
     }
   }
 
@@ -306,10 +355,15 @@ Status KuduTransaction::Data::IsCommitCompleteImpl(
       *is_complete = false;
       *completion_status = Status::IllegalState("transaction is still open");
       break;
+    case TxnStatePB::ABORT_IN_PROGRESS:
+      *is_complete = false;
+      *completion_status = Status::Aborted("transaction is being aborted");
+      break;
     case TxnStatePB::ABORTED:
       *is_complete = true;
       *completion_status = Status::Aborted("transaction has been aborted");
       break;
+    case TxnStatePB::FINALIZE_IN_PROGRESS:
     case TxnStatePB::COMMIT_IN_PROGRESS:
       *is_complete = false;
       *completion_status = Status::Incomplete("commit is still in progress");
@@ -324,6 +378,21 @@ Status KuduTransaction::Data::IsCommitCompleteImpl(
         return Status::IllegalState(errmsg);
       }
   }
+  // Update last observed timestamp -- this is necessary to achieve consistency
+  // in the READ_YOUR_WRITES mode when trying to read back the data of a
+  // transaction that has just been committed. The commit phase might take some
+  // time and may even be retried when the involved participants are not
+  // available, so even the client has observed timestamps for all the write
+  // operations sent in the context this transaction, the maximum timestamp
+  // among all the transaction participants might be far ahead of the latest
+  // timestamp observed by the client while performing the transactional write
+  // operations.
+  // NOTE: an empty transaction doesn't have the commit timestamp assigned.
+  if (resp.has_commit_timestamp()) {
+    DCHECK(state == TxnStatePB::COMMITTED ||
+           state == TxnStatePB::FINALIZE_IN_PROGRESS);
+    client->data_->UpdateLatestObservedTimestamp(resp.commit_timestamp());
+  }
   return Status::OK();
 }
 
@@ -354,6 +423,7 @@ struct KeepaliveRpcCtx {
   unique_ptr<AsyncRandomTxnManagerRpc<KeepTransactionAliveRequestPB,
                                       KeepTransactionAliveResponsePB>> rpc;
   sp::weak_ptr<KuduTransaction> weak_txn;
+  sp::shared_ptr<KuduClient> client;
 };
 
 void KuduTransaction::Data::SendTxnKeepAliveTask(
@@ -383,13 +453,15 @@ void KuduTransaction::Data::SendTxnKeepAliveTask(
   }
 
   const auto& txn_id = txn->data_->txn_id_;
-  const auto next_run_after = MonoDelta::FromMilliseconds(
-      std::max<uint32_t>(1, txn->data_->txn_keep_alive_ms_ / 2));
-  auto deadline = MonoTime::Now() + next_run_after;
+  auto deadline = MonoTime::Now() + txn->data_->GetKeepaliveRpcTimeout();
 
   sp::shared_ptr<KeepaliveRpcCtx> ctx(new KeepaliveRpcCtx);
   ctx->weak_txn = weak_txn;
 
+  // We need to ensure the KuduClient outlives each RPC, so maintain a
+  // reference to the KuduClient until the callback is called.
+  ctx->client = c;
+
   {
     KeepTransactionAliveRequestPB req;
     req.set_txn_id(txn_id);
@@ -437,15 +509,24 @@ void KuduTransaction::Data::TxnKeepAliveCb(
 
   // Re-schedule the task, so it will send another keepalive heartbeat as
   // necessary.
-  sp::shared_ptr<KuduClient> c(txn->data_->weak_client_.lock());
-  if (PREDICT_FALSE(!c)) {
-    return;
-  }
-  auto m = c->data_->messenger_;
+  //
+  // If there was an error with the prior request, send the next one sooner
+  // since one heartbeat has just been missed. If the prior request timed out,
+  // send the next one as soon as possible. It's been a long interval since the
+  // previous successfully sent keepalive message and it's been a long enough
+  // interval since issuing preivous KeepTransactionAlive() RPC, so this should
+  // not put too much pressure on the cluster (assuming the keepalive intervals
+  // for a transaction is in order of a minute). Sending next message sooner
+  // increases the chances of eventually getting through in such a case.
+  const auto next_run_after =
+      s.ok() ? txn->data_->GetKeepaliveRpcPeriod()
+             : (s.IsTimedOut() ? MonoDelta::FromMilliseconds(1)
+                               : txn->data_->GetKeepaliveRpcTimeout());
+  DCHECK_GE(txn->data_->GetKeepaliveRpcPeriod().ToMilliseconds(),
+            txn->data_->GetKeepaliveRpcTimeout().ToMilliseconds());
+  auto m = ctx->client->data_->messenger_;
   if (PREDICT_TRUE(m)) {
     auto weak_txn = ctx->weak_txn;
-    const auto next_run_after = MonoDelta::FromMilliseconds(
-        std::max<uint32_t>(1, txn->data_->txn_keep_alive_ms_ / 2));
     m->ScheduleOnReactor(
         [weak_txn](const Status& s) {
           SendTxnKeepAliveTask(s, weak_txn);
