@@ -18,6 +18,7 @@
 #include "kudu/consensus/log_reader.h"
 
 #include <algorithm>
+#include <cstddef>
 #include <memory>
 #include <mutex>
 #include <ostream>
@@ -245,7 +246,7 @@ scoped_refptr<ReadableLogSegment> LogReader::GetSegmentBySequenceNumber(int64_t
 
 Status LogReader::ReadBatchUsingIndexEntry(const LogIndexEntry& index_entry,
                                            faststring* tmp_buf,
-                                           unique_ptr<LogEntryBatchPB>* batch) const {
+                                           LogEntryBatchPB* batch) const {
   const int64_t index = index_entry.op_id.index();
 
   scoped_refptr<ReadableLogSegment> segment = GetSegmentBySequenceNumber(
@@ -270,7 +271,7 @@ Status LogReader::ReadBatchUsingIndexEntry(const LogIndexEntry& index_entry,
 
   if (bytes_read_) {
     bytes_read_->IncrementBy(segment->entry_header_size() + tmp_buf->length());
-    entries_read_->IncrementBy((**batch).entry_size());
+    entries_read_->IncrementBy(batch->entry_size());
   }
 
   return Status::OK();
@@ -288,10 +289,10 @@ Status LogReader::ReadReplicatesInRange(int64_t starting_at,
   ElementDeleter d(&replicates_tmp);
   LogIndexEntry prev_index_entry;
 
-  int64_t total_size = 0;
+  size_t total_size = 0;
   bool limit_exceeded = false;
   faststring tmp_buf;
-  unique_ptr<LogEntryBatchPB> batch;
+  LogEntryBatchPB batch;
   for (int64_t index = starting_at; index <= up_to && !limit_exceeded; index++) {
     LogIndexEntry index_entry;
     RETURN_NOT_OK_PREPEND(log_index_->GetEntry(index, &index_entry),
@@ -308,21 +309,21 @@ Status LogReader::ReadReplicatesInRange(int64_t starting_at,
 
       // Sanity-check the property that a batch should only have increasing indexes.
       int64_t prev_index = 0;
-      for (int i = 0; i < batch->entry_size(); ++i) {
-        LogEntryPB* entry = batch->mutable_entry(i);
+      for (int i = 0; i < batch.entry_size(); ++i) {
+        LogEntryPB* entry = batch.mutable_entry(i);
         if (!entry->has_replicate()) continue;
         int64_t this_index = entry->replicate().id().index();
         CHECK_GT(this_index, prev_index)
           << "Expected that an entry batch should only include increasing log indexes: "
           << index_entry.ToString()
-          << "\nBatch: " << SecureDebugString(*batch);
+          << "\nBatch: " << SecureDebugString(batch);
         prev_index = this_index;
       }
     }
 
     bool found = false;
-    for (int i = 0; i < batch->entry_size(); ++i) {
-      LogEntryPB* entry = batch->mutable_entry(i);
+    for (int i = 0; i < batch.entry_size(); ++i) {
+      LogEntryPB* entry = batch.mutable_entry(i);
       if (!entry->has_replicate()) {
         continue;
       }
@@ -331,7 +332,7 @@ Status LogReader::ReadReplicatesInRange(int64_t starting_at,
         continue;
       }
 
-      int64_t space_required = entry->replicate().SpaceUsed();
+      size_t space_required = entry->replicate().SpaceUsedLong();
       if (replicates_tmp.empty() ||
           max_bytes_to_read <= 0 ||
           total_size + space_required < max_bytes_to_read) {
