@@ -15,6 +15,8 @@
 // specific language governing permissions and limitations
 // under the License.
 
+#include "kudu/fs/data_dirs.h"
+
 #include <cmath>
 #include <cstdint>
 #include <memory>
@@ -32,7 +34,7 @@
 #include <gtest/gtest.h>
 
 #include "kudu/fs/block_manager.h"
-#include "kudu/fs/data_dirs.h"
+#include "kudu/fs/dir_manager.h"
 #include "kudu/fs/fs.pb.h"
 #include "kudu/gutil/casts.h"
 #include "kudu/gutil/map-util.h"
@@ -89,7 +91,7 @@ class DataDirsTest : public KuduTest {
     DataDirManagerOptions opts;
     opts.metric_entity = entity_;
     ASSERT_OK(DataDirManager::CreateNewForTests(
-        env_, GetDirNames(kNumDirs), std::move(opts), &dd_manager_));
+        env_, GetDirNames(kNumDirs), opts, &dd_manager_));
   }
 
  protected:
@@ -114,7 +116,7 @@ class DataDirsTest : public KuduTest {
 TEST_F(DataDirsTest, TestCreateGroup) {
   // Test that the DataDirManager doesn't know about the tablets we're about
   // to insert.
-  DataDir* dd = nullptr;
+  Dir* dd = nullptr;
   Status s = dd_manager_->GetDirAddIfNecessary(test_block_opts_, &dd);
   ASSERT_EQ(nullptr, dd);
   ASSERT_TRUE(s.IsNotFound()) << s.ToString();
@@ -183,7 +185,7 @@ TEST_F(DataDirsTest, TestLoadFromPB) {
 
 TEST_F(DataDirsTest, TestDeleteDataDirGroup) {
   ASSERT_OK(dd_manager_->CreateDataDirGroup(test_tablet_name_));
-  DataDir* dd;
+  Dir* dd;
   ASSERT_OK(dd_manager_->GetDirAddIfNecessary(test_block_opts_, &dd));
   ASSERT_FALSE(dd->is_full());
   dd_manager_->DeleteDataDirGroup(test_tablet_name_);
@@ -226,7 +228,7 @@ TEST_F(DataDirsTest, TestFullDiskGrowsGroup) {
   FLAGS_env_inject_full = 1.0;
 
   // Try getting a new directory, adding if necessary.
-  DataDir* new_dir;
+  Dir* new_dir;
   ASSERT_OK(dd_manager_->GetDirAddIfNecessary(test_block_opts_, &new_dir));
   unordered_set<string> old_dirs(data_dir_group.begin(), data_dir_group.end());
   ASSERT_FALSE(ContainsKey(old_dirs, new_dir->dir()));
@@ -259,7 +261,7 @@ TEST_F(DataDirsTest, TestGrowGroupInParallel) {
   const int kNumThreads = 32;
   vector<thread> threads;
   vector<Status> statuses(kNumThreads);
-  vector<DataDir*> dds(kNumThreads);
+  vector<Dir*> dds(kNumThreads);
   Barrier b(kNumThreads);
   for (int i = 0; i < kNumThreads; i++) {
     threads.emplace_back([&, i] {
@@ -277,7 +279,7 @@ TEST_F(DataDirsTest, TestGrowGroupInParallel) {
   unordered_set<string> old_dirs(data_dir_group.begin(), data_dir_group.end());
   ASSERT_OK(dd_manager_->FindDataDirsByTabletId(test_tablet_name_, &data_dir_group));
   ASSERT_EQ(FLAGS_fs_target_data_dirs_per_tablet + 1, data_dir_group.size());
-  DataDir* new_dir = dds[0];
+  Dir* new_dir = dds[0];
   ASSERT_FALSE(ContainsKey(old_dirs, new_dir->dir()));
 
   // All returned data directories should have been the newly added one.
@@ -289,16 +291,16 @@ TEST_F(DataDirsTest, TestGrowGroupInParallel) {
 TEST_F(DataDirsTest, TestFailedDirNotReturned) {
   FLAGS_fs_target_data_dirs_per_tablet = 2;
   ASSERT_OK(dd_manager_->CreateDataDirGroup(test_tablet_name_));
-  DataDir* dd;
-  DataDir* failed_dd;
+  Dir* dd;
+  Dir* failed_dd;
   int uuid_idx;
   // Fail one of the directories in the group and verify that it is not used.
   ASSERT_OK(dd_manager_->GetDirAddIfNecessary(test_block_opts_, &failed_dd));
-  ASSERT_TRUE(dd_manager_->FindUuidIndexByDataDir(failed_dd, &uuid_idx));
+  ASSERT_TRUE(dd_manager_->FindUuidIndexByDir(failed_dd, &uuid_idx));
   // These calls are idempotent.
-  ASSERT_OK(dd_manager_->MarkDataDirFailed(uuid_idx));
-  ASSERT_OK(dd_manager_->MarkDataDirFailed(uuid_idx));
-  ASSERT_OK(dd_manager_->MarkDataDirFailed(uuid_idx));
+  ASSERT_OK(dd_manager_->MarkDirFailed(uuid_idx));
+  ASSERT_OK(dd_manager_->MarkDirFailed(uuid_idx));
+  ASSERT_OK(dd_manager_->MarkDirFailed(uuid_idx));
   ASSERT_EQ(1, down_cast<AtomicGauge<uint64_t>*>(
         entity_->FindOrNull(METRIC_data_dirs_failed).get())->value());
   for (int i = 0; i < 10; i++) {
@@ -307,8 +309,8 @@ TEST_F(DataDirsTest, TestFailedDirNotReturned) {
   }
 
   // Fail the other directory and verify that neither will be used.
-  ASSERT_TRUE(dd_manager_->FindUuidIndexByDataDir(dd, &uuid_idx));
-  ASSERT_OK(dd_manager_->MarkDataDirFailed(uuid_idx));
+  ASSERT_TRUE(dd_manager_->FindUuidIndexByDir(dd, &uuid_idx));
+  ASSERT_OK(dd_manager_->MarkDirFailed(uuid_idx));
   ASSERT_EQ(2, down_cast<AtomicGauge<uint64_t>*>(
         entity_->FindOrNull(METRIC_data_dirs_failed).get())->value());
   Status s = dd_manager_->GetDirAddIfNecessary(test_block_opts_, &failed_dd);
@@ -320,7 +322,7 @@ TEST_F(DataDirsTest, TestFailedDirNotAddedToGroup) {
   // Fail one dir and create a group with all directories. The failed directory
   // shouldn't be in the group.
   FLAGS_fs_target_data_dirs_per_tablet = kNumDirs;
-  ASSERT_OK(dd_manager_->MarkDataDirFailed(0));
+  ASSERT_OK(dd_manager_->MarkDirFailed(0));
   ASSERT_EQ(1, down_cast<AtomicGauge<uint64_t>*>(
         entity_->FindOrNull(METRIC_data_dirs_failed).get())->value());
   ASSERT_OK(dd_manager_->CreateDataDirGroup(test_tablet_name_));
@@ -338,10 +340,10 @@ TEST_F(DataDirsTest, TestFailedDirNotAddedToGroup) {
   dd_manager_->DeleteDataDirGroup(test_tablet_name_);
 
   for (int i = 1; i < kNumDirs - 1; i++) {
-    ASSERT_OK(dd_manager_->MarkDataDirFailed(i));
+    ASSERT_OK(dd_manager_->MarkDirFailed(i));
   }
-  Status s = dd_manager_->MarkDataDirFailed(kNumDirs - 1);
-  ASSERT_STR_CONTAINS(s.ToString(), "All data dirs have failed");
+  Status s = dd_manager_->MarkDirFailed(kNumDirs - 1);
+  ASSERT_STR_CONTAINS(s.ToString(), "All dirs have failed");
   ASSERT_TRUE(s.IsIOError());
 
   s = dd_manager_->CreateDataDirGroup(test_tablet_name_);
@@ -365,7 +367,7 @@ TEST_F(DataDirsTest, TestLoadBalancingDistribution) {
   double sum_squared_dev = 0;
   for (const auto& e : dd_manager_->tablets_by_uuid_idx_map_) {
     LOG(INFO) << Substitute("$0 is storing data from $1 tablets.",
-        dd_manager_->data_dir_by_uuid_idx_[e.first]->dir(), e.second.size());
+        dd_manager_->dir_by_uuid_idx_[e.first]->dir(), e.second.size());
     double deviation = static_cast<double>(e.second.size()) - kMeanTabletsPerDir;
     sum_squared_dev += deviation * deviation;
   }
@@ -427,7 +429,7 @@ TEST_F(DataDirsTest, TestLoadBalancingBias) {
       kNumAdditionalTablets * FLAGS_fs_target_data_dirs_per_tablet) / kNumDirs;
   for (const auto& e : dd_manager_->tablets_by_uuid_idx_map_) {
     LOG(INFO) << Substitute("$0 is storing data from $1 tablets.",
-        dd_manager_->data_dir_by_uuid_idx_[e.first]->dir(), e.second.size());
+        dd_manager_->dir_by_uuid_idx_[e.first]->dir(), e.second.size());
     double deviation = static_cast<double>(e.second.size()) - kMeanTabletsPerDir;
     sum_squared_dev += deviation * deviation;
   }
@@ -463,8 +465,7 @@ class DataDirManagerTest : public DataDirsTest {
   }
 
   Status OpenDataDirManager() {
-    return DataDirManager::OpenExistingForTests(env_, test_roots_,
-        DataDirManagerOptions(), &dd_manager_);
+    return DataDirManager::OpenExistingForTests(env_, test_roots_, {}, &dd_manager_);
   }
 
   virtual int GetNumDirs() const { return kNumDirs; }
@@ -482,8 +483,7 @@ TEST_F(DataDirManagerTest, TestOpenWithFailedDirs) {
   for (const string& test_root : test_roots_) {
     ASSERT_OK(env_->CreateDir(test_root));
   }
-  ASSERT_OK(DataDirManager::CreateNewForTests(
-      env_, test_roots_, DataDirManagerOptions(), &dd_manager_));
+  ASSERT_OK(DataDirManager::CreateNewForTests(env_, test_roots_, {}, &dd_manager_));
 
   // Kill the first directory.
   FLAGS_crash_on_eio = false;
@@ -493,7 +493,7 @@ TEST_F(DataDirManagerTest, TestOpenWithFailedDirs) {
   // The directory manager will successfully open with the single failed directory.
   ASSERT_OK(OpenDataDirManager());
   set<int> failed_dirs;
-  ASSERT_EQ(1, dd_manager_->GetFailedDataDirs().size());
+  ASSERT_EQ(1, dd_manager_->GetFailedDirs().size());
 
   // Now fail almost all of the other directories, leaving the last intact.
   for (int i = 1; i < kNumDirs - 1; i++) {
@@ -502,13 +502,13 @@ TEST_F(DataDirManagerTest, TestOpenWithFailedDirs) {
   }
   // The directory manager should be aware of the new failures.
   ASSERT_OK(OpenDataDirManager());
-  ASSERT_EQ(kNumDirs - 1, dd_manager_->GetFailedDataDirs().size());
+  ASSERT_EQ(kNumDirs - 1, dd_manager_->GetFailedDirs().size());
 
   // Ensure that when there are no healthy data directories, the open will
   // yield an error.
   FLAGS_env_inject_eio_globs = JoinStrings(JoinPathSegmentsV(test_roots_, "**"), ",");
   Status s = DataDirManager::OpenExistingForTests(env_, test_roots_,
-      DataDirManagerOptions(), &dd_manager_);
+      {}, &dd_manager_);
   ASSERT_STR_CONTAINS(s.ToString(), "could not open directory manager");
   ASSERT_TRUE(s.IsNotFound());
 }
@@ -526,8 +526,7 @@ TEST_F(TooManyDataDirManagerTest, TestTooManyInternedStrings) {
   for (const auto& r : test_roots_) {
     ASSERT_OK(env_->CreateDir(r));
   }
-  ASSERT_OK(DataDirManager::CreateNewForTests(env_, test_roots_,
-      DataDirManagerOptions(), &dd_manager_));
+  ASSERT_OK(DataDirManager::CreateNewForTests(env_, test_roots_, {}, &dd_manager_));
 }
 
 } // namespace fs
