@@ -17,6 +17,7 @@
 
 #pragma once
 
+#include <atomic>
 #include <cstdint>
 #include <iosfwd>
 #include <memory>
@@ -46,6 +47,7 @@ class BlockId;
 class FileCache;
 class InstanceMetadataPB;
 class MemTracker;
+class Timer;
 
 namespace fs {
 
@@ -181,7 +183,20 @@ class FsManager {
   // If the filesystem has not been initialized, returns NotFound. In that
   // case, CreateInitialFileSystemLayout() may be used to initialize the
   // on-disk and in-memory structures.
-  Status Open(fs::FsReport* report = nullptr);
+  //
+  // If 'read_instance_metadata_files' and 'read_data_directories' are not nullptr,
+  // they will be populated with time spent reading the instance metadata files
+  // and time spent reading data directories respectively.
+  //
+  // If 'containers_processed' and 'containers_total' are not nullptr, they will
+  // be populated with total containers attempted to be opened/processed and
+  // total containers present respectively in the subsequent calls made to
+  // the block manager.
+  Status Open(fs::FsReport* report = nullptr,
+              Timer* read_instance_metadata_files = nullptr,
+              Timer* read_data_directories = nullptr,
+              std::atomic<int>* containers_processed = nullptr,
+              std::atomic<int>* containers_total = nullptr );
 
   // Create the initial filesystem layout. If 'uuid' is provided, uses it as
   // uuid of the filesystem. Otherwise generates one at random.
@@ -219,8 +234,6 @@ class FsManager {
   Status OpenBlock(const BlockId& block_id,
                    std::unique_ptr<fs::ReadableBlock>* block);
 
-  Status DeleteBlock(const BlockId& block_id);
-
   bool BlockExists(const BlockId& block_id) const;
 
   // ==========================================================================
@@ -298,6 +311,10 @@ class FsManager {
   // Prints the file system trees under the file system roots.
   void DumpFileSystemTree(std::ostream& out);
 
+  bool meta_on_xfs() const {
+    return meta_on_xfs_;
+  }
+
  private:
   FRIEND_TEST(fs::FsManagerTestBase, TestDuplicatePaths);
   FRIEND_TEST(fs::FsManagerTestBase, TestEIOWhileRunningUpdateDirsTool);
@@ -366,8 +383,6 @@ class FsManager {
   static const char *kTabletMetadataDirName;
   static const char *kWalDirName;
   static const char *kInstanceMetadataFileName;
-  static const char *kInstanceMetadataMagicNumber;
-  static const char *kTabletSuperBlockMagicNumber;
   static const char *kConsensusMetadataDirName;
 
   // The environment to be used for all filesystem operations.
@@ -396,6 +411,9 @@ class FsManager {
 
   bool initted_;
 
+  // Cache whether or not the metadata directory is on an XFS directory.
+  bool meta_on_xfs_;
+
   DISALLOW_COPY_AND_ASSIGN(FsManager);
 };
 
