@@ -44,6 +44,7 @@
 #include "kudu/gutil/port.h"
 #include "kudu/gutil/stl_util.h"
 #include "kudu/gutil/strings/substitute.h"
+#include "kudu/gutil/strings/util.h"
 #include "kudu/master/master.pb.h"
 #include "kudu/master/master.proxy.h"
 #include "kudu/mini-cluster/external_mini_cluster.h"
@@ -140,7 +141,7 @@ client::KuduSchema SimpleIntKeyKuduSchema() {
 }
 
 Status GetLastOpIdForEachReplica(const string& tablet_id,
-                                 const vector<TServerDetails*>& replicas,
+                                 const vector<const TServerDetails*>& replicas,
                                  OpIdType opid_type,
                                  const MonoDelta& timeout,
                                  vector<OpId>* op_ids) {
@@ -150,7 +151,7 @@ Status GetLastOpIdForEachReplica(const string& tablet_id,
   RpcController controller;
 
   op_ids->clear();
-  for (TServerDetails* ts : replicas) {
+  for (const auto* ts : replicas) {
     controller.Reset();
     controller.set_timeout(timeout);
     opid_resp.Clear();
@@ -168,7 +169,7 @@ Status GetLastOpIdForEachReplica(const string& tablet_id,
 }
 
 Status GetLastOpIdForReplica(const std::string& tablet_id,
-                             TServerDetails* replica,
+                             const TServerDetails* replica,
                              OpIdType opid_type,
                              const MonoDelta& timeout,
                              consensus::OpId* op_id) {
@@ -222,7 +223,7 @@ Status WaitForServersToAgree(const MonoDelta& timeout,
                              consensus::OpIdType op_id_type) {
   const MonoTime deadline = MonoTime::Now() + timeout;
 
-  vector<TServerDetails*> servers;
+  vector<const TServerDetails*> servers;
   AppendValuesFromMap(tablet_servers, &servers);
   for (int i = 1; MonoTime::Now() < deadline; i++) {
     vector<OpId> ids;
@@ -260,45 +261,6 @@ Status WaitForServersToAgree(const MonoDelta& timeout,
                  minimum_index, timeout.ToString()));
 }
 
-// Wait until all specified replicas have logged the given index.
-Status WaitUntilAllReplicasHaveOp(const int64_t log_index,
-                                  const string& tablet_id,
-                                  const vector<TServerDetails*>& replicas,
-                                  const MonoDelta& timeout) {
-  MonoTime start = MonoTime::Now();
-  MonoDelta passed = MonoDelta::FromMilliseconds(0);
-  while (true) {
-    vector<OpId> op_ids;
-    Status s = GetLastOpIdForEachReplica(tablet_id, replicas, consensus::RECEIVED_OPID, timeout,
-                                         &op_ids);
-    if (s.ok()) {
-      bool any_behind = false;
-      for (const OpId& op_id : op_ids) {
-        if (op_id.index() < log_index) {
-          any_behind = true;
-          break;
-        }
-      }
-      if (!any_behind) return Status::OK();
-    } else {
-      LOG(WARNING) << "Got error getting last opid for each replica: " << s.ToString();
-    }
-    passed = MonoTime::Now() - start;
-    if (passed > timeout) {
-      break;
-    }
-    SleepFor(MonoDelta::FromMilliseconds(50));
-  }
-  string replicas_str;
-  for (const TServerDetails* replica : replicas) {
-    if (!replicas_str.empty()) replicas_str += ", ";
-    replicas_str += "{ " + replica->ToString() + " }";
-  }
-  return Status::TimedOut(Substitute("Index $0 not available on all replicas after $1. "
-                                     "Replicas: [ $2 ]",
-                                     log_index, passed.ToString(), replicas_str));
-}
-
 Status CreateTabletServerMap(const shared_ptr<MasterServiceProxy>& master_proxy,
                              const shared_ptr<Messenger>& messenger,
                              unordered_map<string, TServerDetails*>* ts_map) {
@@ -784,10 +746,15 @@ Status RequestVote(const TServerDetails* replica,
 Status LeaderStepDown(const TServerDetails* replica,
                       const string& tablet_id,
                       const MonoDelta& timeout,
-                      TabletServerErrorPB* error) {
+                      TabletServerErrorPB* error,
+                      const std::string& new_leader_uuid) {
   LeaderStepDownRequestPB req;
   req.set_dest_uuid(replica->uuid());
   req.set_tablet_id(tablet_id);
+  if (!new_leader_uuid.empty()) {
+    req.set_new_leader_uuid(new_leader_uuid);
+    req.set_mode(consensus::GRACEFUL);
+  }
   LeaderStepDownResponsePB resp;
   RpcController rpc;
   rpc.set_timeout(timeout);
@@ -1263,6 +1230,8 @@ Status GetInt64Metric(const HostPort& http_hp,
                       const MetricPrototype* metric_proto,
                       const char* value_field,
                       int64_t* value) {
+  *value = 0;
+  bool found = false;
   // Fetch metrics whose name matches the given prototype.
   string url = Substitute(
       "http://$0/jsonmetricz?metrics=$1",
@@ -1286,7 +1255,7 @@ Status GetInt64Metric(const HostPort& http_hp,
     if (entity_id) {
       string id;
       RETURN_NOT_OK(r.ExtractString(entity, "id", &id));
-      if (id != entity_id) {
+      if (!MatchPattern(id, entity_id)) {
         continue;
       }
     }
@@ -1300,10 +1269,19 @@ Status GetInt64Metric(const HostPort& http_hp,
       if (name != metric_proto->name()) {
         continue;
       }
-      RETURN_NOT_OK(r.ExtractInt64(metric, value_field, value));
-      return Status::OK();
+
+      int64_t v = 0;
+      RETURN_NOT_OK(r.ExtractInt64(metric, value_field, &v));
+      found = true;
+      *value += v;
+      if (!entity_id) {
+        return Status::OK();
+      }
     }
   }
+  if (found) {
+    return Status::OK();
+  }
   string msg;
   if (entity_id) {
     msg = Substitute("Could not find metric $0.$1 for entity $2",
