@@ -547,18 +547,17 @@ TEST_P(TsRecoveryITest, TestChangeMaxCellSize) {
                             work.rows_inserted()));
 }
 
-class TsRecoveryITestDeathTest : public TsRecoveryITest {};
+#if !defined(THREAD_SANITIZER)
+// TSAN cannot handle spawning threads after fork(), so this class and its
+// test scenario isn't present for TSAN builds.
+class TsRecoveryITestDeathTest : public TsRecoveryITest {
+};
 
 // Test that tablet bootstrap can automatically repair itself if it finds an
 // overflowed OpId index written to the log caused by KUDU-1933.
 // Also serves as a regression itest for KUDU-1933 by writing ops with a high
 // term and index.
 TEST_P(TsRecoveryITestDeathTest, TestRecoverFromOpIdOverflow) {
-#if defined(THREAD_SANITIZER)
-  // TSAN cannot handle spawning threads after fork().
-  return;
-#endif
-
   // Create the initial tablet files on disk, then shut down the cluster so we
   // can meddle with the WAL.
   NO_FATALS(StartClusterOneTs());
@@ -696,6 +695,7 @@ TEST_P(TsRecoveryITestDeathTest, TestRecoverFromOpIdOverflow) {
   });
   NO_FATALS(cluster_->AssertNoCrashes());
 }
+#endif // #if !defined(THREAD_SANITIZER) ...
 
 // A set of threads which pick rows which are known to exist in the table
 // and issue random updates against them.
@@ -779,13 +779,13 @@ class UpdaterThreads {
 // never hit disk.
 class Kudu969Test : public TsRecoveryITest {
 };
-INSTANTIATE_TEST_CASE_P(DifferentFaultPoints,
-                        Kudu969Test,
-                        ::testing::Values("fault_crash_before_flush_tablet_meta_after_compaction",
+INSTANTIATE_TEST_SUITE_P(DifferentFaultPoints,
+                         Kudu969Test,
+                         ::testing::Values("fault_crash_before_flush_tablet_meta_after_compaction",
                                           "fault_crash_before_flush_tablet_meta_after_flush_mrs"));
 
 TEST_P(Kudu969Test, Test) {
-  if (!AllowSlowTests()) return;
+  SKIP_IF_SLOW_NOT_ALLOWED();
 
   // We use a replicated cluster here so that the 'REPLICATE' messages
   // and 'COMMIT' messages are spread out further in time, and it's
@@ -857,7 +857,7 @@ TEST_P(Kudu969Test, Test) {
 
 // Passes block manager types to the recovery test so we get some extra
 // testing to cover non-default block manager types.
-INSTANTIATE_TEST_CASE_P(BlockManagerType, TsRecoveryITest,
-    ::testing::ValuesIn(BlockManager::block_manager_types()));
+INSTANTIATE_TEST_SUITE_P(BlockManagerType, TsRecoveryITest,
+                         ::testing::ValuesIn(BlockManager::block_manager_types()));
 
 } // namespace kudu
