@@ -155,7 +155,8 @@ AutoRebalancerTask::AutoRebalancerTask(CatalogManager* catalog_manager,
       /*run_policy_fixer*/true,
       /*run_cross_location_rebalancing*/true,
       /*run_intra_location_rebalancing*/true,
-      FLAGS_auto_rebalancing_load_imbalance_threshold))),
+      FLAGS_auto_rebalancing_load_imbalance_threshold,
+      /*force_rebalance_replicas_on_maintenance_tservers*/false))),
       random_generator_(random_device_()),
       number_of_loop_iterations_for_test_(0),
       moves_scheduled_this_round_for_test_(0) {
