@@ -39,7 +39,6 @@
 #include "kudu/util/status.h"
 #include "kudu/util/test_macros.h"
 
-using boost::make_optional;
 using boost::none;
 using boost::optional;
 using std::initializer_list;
@@ -104,7 +103,7 @@ class PlacementPolicyTest : public ::testing::Test {
     for (const auto& location_info : cluster_info) {
       const auto& ts_infos = location_info.tablet_servers;
       for (const auto& ts : ts_infos) {
-        shared_ptr<TSDescriptor> tsd(new TSDescriptor(ts.id));
+        auto tsd(TSDescriptor::make_shared(ts.id));
         tsd->set_num_live_replicas(ts.replica_num);
         tsd->location_.emplace(location_info.id);
         tsd->set_num_live_replicas_by_dimension(ts.replica_num_by_dimension);
@@ -1011,7 +1010,7 @@ TEST_F(PlacementPolicyTest, PlaceTabletReplicasWithNewTabletServers) {
       map<string, int> placement_stats;
       for (auto i = 0; i < 1000; ++i) {
         TSDescriptorVector result;
-        ASSERT_OK(policy.PlaceTabletReplicas(3, make_optional(string(label)), &result));
+        ASSERT_OK(policy.PlaceTabletReplicas(3, boost::make_optional(string(label)), &result));
         ASSERT_EQ(3, result.size());
         for (const auto& ts : result) {
           const auto& ts_uuid = ts->permanent_uuid();
