@@ -53,7 +53,6 @@
 #include "kudu/gutil/strings/strip.h"
 #include "kudu/gutil/strings/substitute.h"
 #include "kudu/security/gssapi.h"
-#include "kudu/security/openssl_util.h"
 #include "kudu/util/easy_json.h"
 #include "kudu/util/env.h"
 #include "kudu/util/flag_tags.h"
@@ -61,6 +60,7 @@
 #include "kudu/util/logging.h"
 #include "kudu/util/net/net_util.h"
 #include "kudu/util/net/sockaddr.h"
+#include "kudu/util/openssl_util.h"
 #include "kudu/util/string_case.h"
 #include "kudu/util/url-coding.h"
 #include "kudu/util/version_info.h"
@@ -72,6 +72,7 @@ typedef sig_t sighandler_t;
 
 using mustache::RenderTemplate;
 using std::ostringstream;
+using std::pair;
 using std::stringstream;
 using std::string;
 using std::unordered_set;
@@ -103,6 +104,8 @@ string HttpStatusCodeToString(kudu::HttpStatusCode code) {
   switch (code) {
     case kudu::HttpStatusCode::Ok:
       return "200 OK";
+    case kudu::HttpStatusCode::TemporaryRedirect:
+      return "307 Temporary Redirect";
     case kudu::HttpStatusCode::BadRequest:
       return "400 Bad Request";
     case kudu::HttpStatusCode::AuthenticationRequired:
@@ -175,7 +178,8 @@ Status RunSpnegoStep(const char* authz_header,
 
 Webserver::Webserver(const WebserverOptions& opts)
   : opts_(opts),
-    context_(nullptr) {
+  context_(nullptr),
+  is_started_(false) {
   string host = opts.bind_interface.empty() ? "0.0.0.0" : opts.bind_interface;
   http_address_ = host + ":" + std::to_string(opts.port);
 }
@@ -185,17 +189,22 @@ Webserver::~Webserver() {
   STLDeleteValues(&path_handlers_);
 }
 
-void Webserver::RootHandler(const WebRequest& /* args */,
+void Webserver::RootHandler(const WebRequest& req,
                             WebResponse* resp) {
-  EasyJson path_handlers = resp->output.Set("path_handlers", EasyJson::kArray);
-  for (const PathHandlerMap::value_type& handler : path_handlers_) {
-    if (handler.second->is_on_nav_bar()) {
-      EasyJson path_handler = path_handlers.PushBack(EasyJson::kObject);
-      path_handler["path"] = handler.first;
-      path_handler["alias"] = handler.second->alias();
+  if (is_started_) {
+    EasyJson path_handlers = resp->output.Set("path_handlers", EasyJson::kArray);
+    for (const PathHandlerMap::value_type& handler : path_handlers_) {
+      if (handler.second->is_on_nav_bar()) {
+        EasyJson path_handler = path_handlers.PushBack(EasyJson::kObject);
+        path_handler["path"] = handler.first;
+        path_handler["alias"] = handler.second->alias();
+      }
     }
+    resp->output["version_info"] = EscapeForHtmlToString(VersionInfo::GetAllVersionInfo());
+  } else {
+    resp->status_code = HttpStatusCode::TemporaryRedirect;
+    resp->response_headers.insert({"Location", "/startup"});
   }
-  resp->output["version_info"] = EscapeForHtmlToString(VersionInfo::GetAllVersionInfo());
 }
 
 void Webserver::BuildArgumentMap(const string& args, ArgumentMap* output) {
@@ -421,6 +430,12 @@ Status Webserver::GetBoundAddresses(std::vector<Sockaddr>* addrs) const {
   return Status::OK();
 }
 
+Status Webserver::GetBoundHostPorts(std::vector<HostPort>* hostports) const {
+  vector<Sockaddr> addrs;
+  RETURN_NOT_OK_PREPEND(GetBoundAddresses(&addrs), "could not get bound webserver addresses");
+  return HostPortsFromAddrs(addrs, hostports);
+}
+
 Status Webserver::GetAdvertisedAddresses(vector<Sockaddr>* addresses) const {
   if (!context_) {
     return Status::ServiceUnavailable("Not started");
@@ -432,6 +447,12 @@ Status Webserver::GetAdvertisedAddresses(vector<Sockaddr>* addresses) const {
   return Status::OK();
 }
 
+Status Webserver::GetAdvertisedHostPorts(vector<HostPort>* hostports) const {
+  vector<Sockaddr> addrs;
+  RETURN_NOT_OK_PREPEND(GetAdvertisedAddresses(&addrs), "could not get bound webserver addresses");
+  return HostPortsFromAddrs(addrs, hostports);
+}
+
 int Webserver::LogMessageCallbackStatic(const struct sq_connection* /*connection*/,
                                         const char* message) {
   if (message != nullptr) {
@@ -699,12 +720,16 @@ void Webserver::RegisterPathHandler(const string& path, const string& alias,
   auto wrapped_cb = [=](const WebRequest& req, PrerenderedWebResponse* rendered_resp) {
     WebResponse resp;
     callback(req, &resp);
-    stringstream out;
     AddKnoxVariables(req, &resp.output);
-    Render(render_path, resp.output, is_styled, &out);
     rendered_resp->status_code = resp.status_code;
     rendered_resp->response_headers = std::move(resp.response_headers);
-    rendered_resp->output << out.rdbuf();
+    // As the home page is redirected to startup until the server's initialization is complete,
+    // do not render the page
+    if (render_path != "/home" || is_started_) {
+      stringstream out;
+      Render(render_path, resp.output, is_styled, &out);
+      rendered_resp->output << out.rdbuf();
+    }
   };
   RegisterPrerenderedPathHandler(path, alias, wrapped_cb, is_styled, is_on_nav_bar);
 }
@@ -782,17 +807,22 @@ void Webserver::RenderMainTemplate(
   ej["static_pages_available"] = static_pages_available();
   ej["content"] = content;
   AddKnoxVariables(req, &ej);
+  std::vector<pair<string, PathHandler*>> paths_and_handlers;
 
   {
     shared_lock<RWMutex> l(lock_);
     ej["footer_html"] = footer_html_;
+    paths_and_handlers.reserve(path_handlers_.size());
+    for (const auto& [path, handler] : path_handlers_) {
+      paths_and_handlers.emplace_back(path, handler);
+    }
   }
   EasyJson path_handlers = ej.Set("path_handlers", EasyJson::kArray);
-  for (const PathHandlerMap::value_type& handler : path_handlers_) {
-    if (handler.second->is_on_nav_bar()) {
+  for (const auto& [path, handler] : paths_and_handlers) {
+    if (handler->is_on_nav_bar()) {
       EasyJson path_handler = path_handlers.PushBack(EasyJson::kObject);
-      path_handler["path"] = handler.first;
-      path_handler["alias"] = handler.second->alias();
+      path_handler["path"] = path;
+      path_handler["alias"] = handler->alias();
     }
   }
   RenderTemplate(kMainTemplate, opts_.doc_root, ej.value(), output);
@@ -818,4 +848,8 @@ void Webserver::set_footer_html(const std::string& html) {
   footer_html_ = html;
 }
 
+void Webserver::SetStartupComplete(bool state) {
+  is_started_ = state;
+}
+
 } // namespace kudu
