@@ -503,6 +503,10 @@ class Tablet {
 
   std::string LogPrefix() const;
 
+  // Return false if the tablets need to compact,
+  // otherwise return true.
+  bool disable_compaction() const;
+
   // Return the default bloom filter sizing parameters, configured by server flags.
   static BloomFilterSizing DefaultBloomSizing();
 
