@@ -60,7 +60,7 @@ class RowSetMetadata;
 class TxnMetadata;
 enum TxnState : int8_t;
 
-typedef std::vector<std::shared_ptr<RowSetMetadata> > RowSetMetadataVector;
+typedef std::vector<std::shared_ptr<RowSetMetadata>> RowSetMetadataVector;
 typedef std::unordered_set<int64_t> RowSetMetadataIds;
 
 // A transaction ID whose MRS is being flushed.
@@ -69,7 +69,7 @@ typedef std::unordered_set<int64_t> RowSetMetadataIds;
 // 'last_durable_mrs_id' per transaction.
 typedef int64_t TxnInfoBeingFlushed;
 
-extern const int64_t kNoDurableMemStore;
+constexpr int64_t kNoDurableMemStore = -1;
 
 // Manages the "blocks tracking" for the specified tablet.
 //
@@ -161,7 +161,7 @@ class TabletMetadata : public RefCountedThreadSafe<TabletMetadata> {
   // even if the schema is changed.
   const Schema& schema() const {
     const Schema* s = reinterpret_cast<const Schema*>(
-        base::subtle::Acquire_Load(reinterpret_cast<const AtomicWord*>(&schema_)));
+        base::subtle::Acquire_Load(reinterpret_cast<const AtomicWord*>(&schema_ptr_)));
     return *s;
   }
 
@@ -290,7 +290,8 @@ class TabletMetadata : public RefCountedThreadSafe<TabletMetadata> {
 
   const RowSetMetadataVector& rowsets() const { return rowsets_; }
 
-  FsManager *fs_manager() const { return fs_manager_; }
+  const FsManager* fs_manager() const { return fs_manager_; }
+  FsManager* fs_manager() { return fs_manager_; }
 
   int64_t last_durable_mrs_id() const { return last_durable_mrs_id_; }
 
@@ -323,18 +324,23 @@ class TabletMetadata : public RefCountedThreadSafe<TabletMetadata> {
     return on_disk_size_.load(std::memory_order_relaxed);
   }
 
+  bool supports_live_row_count() const {
+    return supports_live_row_count_;
+  }
+
+  // Return standard "T xxx P yyy" log prefix.
+  const std::string& LogPrefix() const {
+    return log_prefix_;
+  }
+
   // ==========================================================================
   // Stuff used by the tests
   // ==========================================================================
-  const RowSetMetadata *GetRowSetForTests(int64_t id) const;
-
-  RowSetMetadata *GetRowSetForTests(int64_t id);
+  const RowSetMetadata* GetRowSetForTests(int64_t id) const;
+  RowSetMetadata* GetRowSetForTests(int64_t id);
 
   std::unordered_map<int64_t, scoped_refptr<TxnMetadata>> GetTxnMetadata() const;
 
-  // Return standard "T xxx P yyy" log prefix.
-  std::string LogPrefix() const;
-
   int flush_count_for_tests() const {
     return flush_count_for_tests_;
   }
@@ -344,14 +350,9 @@ class TabletMetadata : public RefCountedThreadSafe<TabletMetadata> {
     supports_live_row_count_ = supports_live_row_count;
   }
 
-  bool supports_live_row_count() const {
-    return supports_live_row_count_;
-  }
-
  private:
-  friend class RefCountedThreadSafe<TabletMetadata>;
   friend class MetadataTest;
-  friend class TestTabletMetadataBenchmark;
+  friend class RefCountedThreadSafe<TabletMetadata>;
 
   // Compile time assert that no one deletes TabletMetadata objects.
   ~TabletMetadata();
@@ -383,7 +384,7 @@ class TabletMetadata : public RefCountedThreadSafe<TabletMetadata> {
 
   // Fully replace superblock.
   // Requires 'flush_lock_'.
-  Status ReplaceSuperBlockUnlocked(const TabletSuperBlockPB &pb);
+  Status ReplaceSuperBlockUnlocked(const TabletSuperBlockPB& pb);
 
   // Requires 'data_lock_'.
   Status UpdateUnlocked(const RowSetMetadataIds& to_remove,
@@ -427,6 +428,7 @@ class TabletMetadata : public RefCountedThreadSafe<TabletMetadata> {
   Partition partition_;
 
   FsManager* const fs_manager_;
+  const std::string log_prefix_;
   RowSetMetadataVector rowsets_;
 
   base::subtle::Atomic64 next_rowset_idx_;
@@ -440,9 +442,11 @@ class TabletMetadata : public RefCountedThreadSafe<TabletMetadata> {
   // merged with main state.
   std::unordered_map<int64_t, scoped_refptr<TxnMetadata>> txn_metadata_by_txn_id_;;
 
-  // The current schema version. This is owned by this class.
-  // We don't use unique_ptr so that we can do an atomic swap.
-  Schema* schema_;
+  // The current schema version.
+  std::unique_ptr<Schema> schema_;
+  // The raw pointer to the schema for an atomic swap.
+  Schema* schema_ptr_;
+
   uint32_t schema_version_;
   std::string table_name_;
   PartitionSchema partition_schema_;
@@ -452,7 +456,7 @@ class TabletMetadata : public RefCountedThreadSafe<TabletMetadata> {
   // a given tablet won't have thousands of "alter table" calls.
   // They are kept alive so that callers of schema() don't need to
   // worry about reference counting or locking.
-  std::vector<Schema*> old_schemas_;
+  std::vector<std::unique_ptr<Schema>> old_schemas_;
 
   // Protected by 'data_lock_'.
   BlockIdSet orphaned_blocks_;
