@@ -40,6 +40,10 @@ class TabletOpBase : public MaintenanceOp {
   TabletOpBase(std::string name, IOUsage io_usage, Tablet* tablet);
   std::string LogPrefix() const;
 
+  // Return false if the tablets need to compact,
+  // otherwise return true.
+  bool DisableCompaction() const;
+
  protected:
   int32_t priority() const override;
 
