@@ -20,7 +20,6 @@
 #include <algorithm>
 #include <cstddef>
 #include <cstdint>
-#include <functional>
 #include <iostream>
 #include <iterator>
 #include <map>
@@ -37,19 +36,15 @@
 #include <boost/optional/optional.hpp>
 #include <glog/logging.h>
 
-#include "kudu/common/wire_protocol.h"
-#include "kudu/common/wire_protocol.pb.h"
 #include "kudu/gutil/basictypes.h"
 #include "kudu/gutil/map-util.h"
 #include "kudu/gutil/port.h"
 #include "kudu/gutil/strings/substitute.h"
 #include "kudu/master/master.pb.h"
-#include "kudu/master/master.proxy.h"
 #include "kudu/rebalance/cluster_status.h"
 #include "kudu/rebalance/placement_policy_util.h"
 #include "kudu/rebalance/rebalance_algo.h"
 #include "kudu/rebalance/rebalancer.h"
-#include "kudu/rpc/response_callback.h"
 #include "kudu/tools/ksck.h"
 #include "kudu/tools/ksck_remote.h"
 #include "kudu/tools/ksck_results.h"
@@ -62,9 +57,7 @@ using kudu::cluster_summary::ServerHealth;
 using kudu::cluster_summary::ServerHealthSummary;
 using kudu::cluster_summary::TableSummary;
 using kudu::cluster_summary::TabletSummary;
-using kudu::master::ListTabletServersRequestPB;
-using kudu::master::ListTabletServersResponsePB;
-using kudu::master::MasterServiceProxy;
+using kudu::master::TServerStatePB;
 using kudu::rebalance::BuildTabletExtraInfoMap;
 using kudu::rebalance::ClusterInfo;
 using kudu::rebalance::ClusterRawInfo;
@@ -110,9 +103,6 @@ Status RebalancerTool::PrintStats(ostream& out) {
   ClusterInfo ci;
   RETURN_NOT_OK(BuildClusterInfo(raw_info, MovesInProgress(), &ci));
 
-  // Print information about replica count of healthy ignored tservers.
-  RETURN_NOT_OK(PrintIgnoredTserversStats(ci, out));
-
   const auto& ts_id_by_location = ci.locality.servers_by_location;
   if (ts_id_by_location.empty()) {
     // Nothing to report about: there are no tablet servers reported.
@@ -120,6 +110,9 @@ Status RebalancerTool::PrintStats(ostream& out) {
     return Status::OK();
   }
 
+  // Print information about tservers need to empty.
+  RETURN_NOT_OK(PrintIgnoredTserversStats(raw_info, out));
+
   if (ts_id_by_location.size() == 1) {
     // That's about printing information about the whole cluster.
     return PrintLocationBalanceStats(ts_id_by_location.begin()->first,
@@ -169,8 +162,7 @@ Status RebalancerTool::Run(RunStatus* result_status, size_t* moves_count) {
   }
 
   ClusterRawInfo raw_info;
-  RETURN_NOT_OK(
-      KsckResultsToClusterRawInfo(boost::none, ksck_->results(), &raw_info));
+  RETURN_NOT_OK(KsckResultsToClusterRawInfo(boost::none, ksck_->results(), &raw_info));
 
   ClusterInfo ci;
   RETURN_NOT_OK(BuildClusterInfo(raw_info, MovesInProgress(), &ci));
@@ -189,7 +181,6 @@ Status RebalancerTool::Run(RunStatus* result_status, size_t* moves_count) {
   size_t moves_count_total = 0;
   if (config_.move_replicas_from_ignored_tservers) {
     // Move replicas from healthy ignored tservers to other healthy tservers.
-    RETURN_NOT_OK(CheckIgnoredServers(raw_info, ci));
     LOG(INFO) << "replacing replicas on healthy ignored tservers";
     IgnoredTserversRunner runner(
         this, config_.ignored_tservers, config_.max_moves_per_server, deadline);
@@ -264,15 +255,28 @@ Status RebalancerTool::Run(RunStatus* result_status, size_t* moves_count) {
   return Status::OK();
 }
 
-Status RebalancerTool::KsckResultsToClusterRawInfo(
-    const boost::optional<string>& location,
-    const KsckResults& ksck_info,
-    ClusterRawInfo* raw_info) {
+Status RebalancerTool::KsckResultsToClusterRawInfo(const boost::optional<string>& location,
+                                                   const KsckResults& ksck_info,
+                                                   ClusterRawInfo* raw_info) {
   DCHECK(raw_info);
+  const auto& cluster_status = ksck_info.cluster_status;
+
+  // Check whether all ignored tservers in the config are valid.
+  if (!config_.ignored_tservers.empty()) {
+    unordered_set<string> known_tservers;
+    for (const auto& ts_summary : ksck_info.cluster_status.tserver_summaries) {
+      known_tservers.emplace(ts_summary.uuid);
+    }
+    for (const auto& ts : config_.ignored_tservers) {
+      if (!ContainsKey(known_tservers, ts)) {
+        return Status::InvalidArgument(
+            Substitute("ignored tserver $0 is not reported among known tservers", ts));
+      }
+    }
+  }
 
   // Filter out entities that are not relevant to the specified location.
   vector<ServerHealthSummary> tserver_summaries;
-  const auto& cluster_status = ksck_info.cluster_status;
   tserver_summaries.reserve(cluster_status.tserver_summaries.size());
 
   vector<TabletSummary> tablet_summaries;
@@ -331,23 +335,35 @@ Status RebalancerTool::KsckResultsToClusterRawInfo(
     }
   }
 
+  unordered_set<string> tservers_in_maintenance_mode;
+  for (const auto& ts : tserver_summaries) {
+    if (ContainsKeyValuePair(ksck_info.ts_states, ts.uuid, TServerStatePB::MAINTENANCE_MODE)) {
+      tservers_in_maintenance_mode.emplace(ts.uuid);
+    }
+  }
+
   raw_info->tserver_summaries = std::move(tserver_summaries);
   raw_info->table_summaries = std::move(table_summaries);
   raw_info->tablet_summaries = std::move(tablet_summaries);
+  raw_info->tservers_in_maintenance_mode = std::move(tservers_in_maintenance_mode);
 
   return Status::OK();
 }
 
-Status RebalancerTool::PrintIgnoredTserversStats(const ClusterInfo& ci,
+Status RebalancerTool::PrintIgnoredTserversStats(const ClusterRawInfo& raw_info,
                                                  ostream& out) const {
-  const auto& tservers_to_empty = ci.tservers_to_empty;
-  if (tservers_to_empty.empty()) {
+  if (config_.ignored_tservers.empty() || !config_.move_replicas_from_ignored_tservers) {
     return Status::OK();
   }
+  unordered_set<string> tservers_to_empty;
+  GetTServersToEmpty(raw_info, &tservers_to_empty);
+  TServersToEmptyMap tservers_to_empty_map;
+  BuildTServersToEmptyInfo(raw_info, MovesInProgress(), tservers_to_empty, &tservers_to_empty_map);
   out << "Per-server replica distribution summary for tservers_to_empty:" << endl;
   DataTable summary({"Server UUID", "Replica Count"});
-  for (const auto& elem: tservers_to_empty) {
-    summary.AddRow({ elem.first, to_string(elem.second) });
+  for (const auto& ts : tservers_to_empty) {
+    auto* tablets = FindOrNull(tservers_to_empty_map, ts);
+    summary.AddRow({ts, tablets ? to_string(tablets->size()) : "0"});
   }
   RETURN_NOT_OK(summary.PrintTo(out));
   out << endl;
@@ -575,9 +591,9 @@ Status RebalancerTool::RunWith(Runner* runner, RunStatus* result_status) {
       break;
     }
 
-    auto has_errors = false;
+    bool has_errors = false;
     while (!is_timed_out) {
-      auto is_scheduled = runner->ScheduleNextMove(&has_errors, &is_timed_out);
+      bool is_scheduled = runner->ScheduleNextMove(&has_errors, &is_timed_out);
       resync_state |= has_errors;
       if (resync_state || is_timed_out) {
         break;
@@ -597,10 +613,9 @@ Status RebalancerTool::RunWith(Runner* runner, RunStatus* result_status) {
       // Poll for the status of pending operations. If some of the in-flight
       // operations are completed, it might be possible to schedule new ones
       // by calling Runner::ScheduleNextMove().
-      auto has_pending_moves = false;
-      auto has_updates = runner->UpdateMovesInProgressStatus(&has_errors,
-                                                             &is_timed_out,
-                                                             &has_pending_moves);
+      bool has_pending_moves = false;
+      bool has_updates =
+          runner->UpdateMovesInProgressStatus(&has_errors, &is_timed_out, &has_pending_moves);
       if (has_updates) {
         // Reset the start of the staleness interval: there was some updates
         // on the status of scheduled move operations.
@@ -644,22 +659,6 @@ Status RebalancerTool::RefreshKsckResults() {
   return Status::OK();
 }
 
-Status RebalancerTool::CheckIgnoredServers(const rebalance::ClusterRawInfo& raw_info,
-                                           const rebalance::ClusterInfo& cluster_info) {
-  int remaining_tservers_count = cluster_info.balance.servers_by_total_replica_count.size();
-  int max_replication_factor = 0;
-  for (const auto& s : raw_info.table_summaries) {
-    max_replication_factor = std::max(max_replication_factor, s.replication_factor);
-  }
-  if (remaining_tservers_count < max_replication_factor) {
-    return Status::InvalidArgument(
-        Substitute("Too many ignored tservers; "
-                   "$0 healthy non-ignored servers exist but $1 are required.",
-                   remaining_tservers_count, max_replication_factor));
-  }
-  return Status::OK();
-}
-
 RebalancerTool::BaseRunner::BaseRunner(RebalancerTool* rebalancer,
                                        std::unordered_set<std::string> ignored_tservers,
                                        size_t max_moves_per_server,
@@ -724,6 +723,38 @@ void RebalancerTool::BaseRunner::UpdateOnMoveCompleted(const string& ts_uuid) {
   DCHECK(ts_per_op_count_updated);
 }
 
+Status RebalancerTool::BaseRunner::CheckTabletServers(const ClusterRawInfo& raw_info) {
+  // For simplicity, allow to run the rebalancing only when all tablet servers
+  // are in good shape (except those specified in 'ignored_tservers').
+  // Otherwise, the rebalancing might interfere with the
+  // automatic re-replication or get unexpected errors while moving replicas.
+  for (const auto& s : raw_info.tserver_summaries) {
+    if (s.health != ServerHealth::HEALTHY && !ContainsKey(ignored_tservers_, s.uuid)) {
+      return Status::IllegalState(Substitute("tablet server $0 ($1): unacceptable health status $2",
+                                             s.uuid,
+                                             s.address,
+                                             ServerHealthToString(s.health)));
+    }
+  }
+  if (rebalancer_->config_.force_rebalance_replicas_on_maintenance_tservers) {
+    return Status::OK();
+  }
+  // Avoid moving replicas to tablet servers that are set maintenance mode.
+  for (const string& ts_uuid : raw_info.tservers_in_maintenance_mode) {
+    if (!ContainsKey(ignored_tservers_, ts_uuid)) {
+      return Status::IllegalState(
+          Substitute("tablet server $0: unacceptable state MAINTENANCE_MODE.\n"
+                     "You can continue rebalancing in one of the following ways:\n"
+                     "1. Set the tserver uuid into the '--ignored_tservers' flag to ignored it.\n"
+                     "2. Set '--force_rebalance_replicas_on_maintenance_tservers' to force "
+                     "rebalancing replicas among all known tservers.\n"
+                     "3. Exit maintenance mode on the tserver.",
+                     ts_uuid));
+    }
+  }
+  return Status::OK();
+}
+
 RebalancerTool::AlgoBasedRunner::AlgoBasedRunner(
     RebalancerTool* rebalancer,
     std::unordered_set<std::string> ignored_tservers,
@@ -926,23 +957,7 @@ Status RebalancerTool::AlgoBasedRunner::GetNextMovesImpl(
   const auto& loc = location();
   ClusterRawInfo raw_info;
   RETURN_NOT_OK(rebalancer_->GetClusterRawInfo(loc, &raw_info));
-
-  // For simplicity, allow to run the rebalancing only when all tablet servers
-  // are in good shape (except those specified in 'ignored_tservers_').
-  // Otherwise, the rebalancing might interfere with the
-  // automatic re-replication or get unexpected errors while moving replicas.
-  for (const auto& s : raw_info.tserver_summaries) {
-    if (s.health != ServerHealth::HEALTHY) {
-      if (ContainsKey(ignored_tservers_, s.uuid)) {
-        LOG(INFO) << Substitute("ignoring unhealthy tablet server $0 ($1).",
-                                s.uuid, s.address);
-        continue;
-      }
-      return Status::IllegalState(
-          Substitute("tablet server $0 ($1): unacceptable health status $2",
-                     s.uuid, s.address, ServerHealthToString(s.health)));
-    }
-  }
+  RETURN_NOT_OK(CheckTabletServers(raw_info));
 
   TabletsPlacementInfo tpi;
   if (!loc) {
@@ -1238,24 +1253,8 @@ Status RebalancerTool::ReplaceBasedRunner::GetNextMovesImpl(
     vector<Rebalancer::ReplicaMove>* replica_moves) {
   ClusterRawInfo raw_info;
   RETURN_NOT_OK(rebalancer_->GetClusterRawInfo(boost::none, &raw_info));
+  RETURN_NOT_OK(CheckTabletServers(raw_info));
 
-  // For simplicity, allow to run the rebalancing only when all tablet servers
-  // are in good shape (except those specified in 'ignored_tservers_').
-  // Otherwise, the rebalancing might interfere with the
-  // automatic re-replication or get unexpected errors while moving replicas.
-  // TODO(aserbin): move it somewhere else?
-  for (const auto& s : raw_info.tserver_summaries) {
-    if (s.health != ServerHealth::HEALTHY) {
-      if (ContainsKey(ignored_tservers_, s.uuid)) {
-        LOG(INFO) << Substitute("ignoring unhealthy tablet server $0 ($1).",
-                                s.uuid, s.address);
-        continue;
-      }
-      return Status::IllegalState(
-          Substitute("tablet server $0 ($1): unacceptable health status $2",
-                     s.uuid, s.address, ServerHealthToString(s.health)));
-    }
-  }
   ClusterInfo ci;
   RETURN_NOT_OK(rebalancer_->BuildClusterInfo(raw_info, scheduled_moves_, &ci));
   return GetReplaceMoves(ci, raw_info, replica_moves);
@@ -1381,82 +1380,52 @@ Status RebalancerTool::IgnoredTserversRunner::GetReplaceMoves(
     const rebalance::ClusterInfo& ci,
     const rebalance::ClusterRawInfo& raw_info,
     vector<Rebalancer::ReplicaMove>* replica_moves) {
+  unordered_set<string> tservers_to_empty;
+  rebalancer_->GetTServersToEmpty(raw_info, &tservers_to_empty);
+  RETURN_NOT_OK(CheckIgnoredTServers(raw_info, ci, tservers_to_empty));
 
-  // In order not to place any replica on the ignored tservers,
-  // allow to run only when all healthy ignored tservers are in
-  // maintenance (or decommision) mode.
-  RETURN_NOT_OK(CheckIgnoredTserversState(ci));
+  TServersToEmptyMap tservers_to_empty_map;
+  BuildTServersToEmptyInfo(raw_info, scheduled_moves_, tservers_to_empty, &tservers_to_empty_map);
 
-  // Build IgnoredTserversInfo.
-  IgnoredTserversInfo ignored_tservers_info;
-  for (const auto& tablet_summary : raw_info.tablet_summaries) {
-    if (ContainsKey(scheduled_moves_, tablet_summary.id)) {
-      continue;
-    }
-    if (tablet_summary.result != cluster_summary::HealthCheckResult::HEALTHY &&
-        tablet_summary.result != cluster_summary::HealthCheckResult::RECOVERING) {
-      LOG(INFO) << Substitute("tablet $0: not considering replicas for movement "
-                              "since the tablet's status is '$1'",
-                              tablet_summary.id,
-                              cluster_summary::HealthCheckResultToString(tablet_summary.result));
-      continue;
-    }
-    TabletInfo tablet_info;
-    for (const auto& replica_info : tablet_summary.replicas) {
-      if (replica_info.is_leader && replica_info.consensus_state) {
-        const auto& cstate = *replica_info.consensus_state;
-        if (cstate.opid_index) {
-          tablet_info.tablet_id = tablet_summary.id;
-          tablet_info.config_idx = *cstate.opid_index;
-          break;
-        }
-      }
-    }
-    for (const auto& replica_info : tablet_summary.replicas) {
-      if (!ContainsKey(ci.tservers_to_empty, replica_info.ts_uuid)) {
-        continue;
-      }
-      auto& tablets = LookupOrEmplace(
-          &ignored_tservers_info, replica_info.ts_uuid, vector<TabletInfo>());
-      tablets.emplace_back(tablet_info);
-    }
-  }
-  GetMovesFromIgnoredTservers(ignored_tservers_info, replica_moves);
+  GetMovesFromIgnoredTservers(tservers_to_empty_map, replica_moves);
   return Status::OK();
 }
 
-Status RebalancerTool::IgnoredTserversRunner::CheckIgnoredTserversState(
-    const rebalance::ClusterInfo& ci) {
-  if (ci.tservers_to_empty.empty()) {
+Status RebalancerTool::IgnoredTserversRunner::CheckIgnoredTServers(
+    const ClusterRawInfo& raw_info,
+    const ClusterInfo& ci,
+    const unordered_set<string>& tservers_to_empty) {
+  if (tservers_to_empty.empty()) {
     return Status::OK();
   }
 
-  LeaderMasterProxy proxy(client_);
-  ListTabletServersRequestPB req;
-  ListTabletServersResponsePB resp;
-  req.set_include_states(true);
-  RETURN_NOT_OK((proxy.SyncRpc<ListTabletServersRequestPB, ListTabletServersResponsePB>(
-      req, &resp, "ListTabletServers", &MasterServiceProxy::ListTabletServersAsync)));
-  if (resp.has_error()) {
-    return StatusFromPB(resp.error().status());
+  // Check whether it is possible to move replicas emptying some tablet servers.
+  int remaining_tservers_count = ci.balance.servers_by_total_replica_count.size();
+  int max_replication_factor = 0;
+  for (const auto& s : raw_info.table_summaries) {
+    max_replication_factor = std::max(max_replication_factor, s.replication_factor);
+  }
+  if (remaining_tservers_count < max_replication_factor) {
+    return Status::InvalidArgument(
+        Substitute("Too many ignored tservers; "
+                   "$0 healthy non-ignored servers exist but $1 are required.",
+                   remaining_tservers_count,
+                   max_replication_factor));
   }
 
-  const auto& servers = resp.servers();
-  for (const auto& server : servers) {
-    const auto& ts_uuid = server.instance_id().permanent_uuid();
-    if (!ContainsKey(ci.tservers_to_empty, ts_uuid)) {
-      continue;
-    }
-    if (server.state() != master::TServerStatePB::MAINTENANCE_MODE) {
-      return Status::IllegalState(Substitute(
-        "You should set maintenance mode for tablet server $0 first", ts_uuid));
+  // Make sure tablet servers that we need to empty are set maintenance mode.
+  for (const auto& ts : tservers_to_empty) {
+    if (!ContainsKey(raw_info.tservers_in_maintenance_mode, ts)) {
+      return Status::IllegalState(
+          Substitute("You should set maintenance mode for tablet server $0 first", ts));
     }
   }
+
   return Status::OK();
 }
 
 void RebalancerTool::IgnoredTserversRunner::GetMovesFromIgnoredTservers(
-    const IgnoredTserversInfo& ignored_tservers_info,
+    const TServersToEmptyMap& ignored_tservers_info,
     vector<Rebalancer::ReplicaMove>* replica_moves) {
   DCHECK(replica_moves);
   if (ignored_tservers_info.empty()) {
