@@ -97,6 +97,10 @@ class RebalancerTool : public rebalance::Rebalancer {
     // (i.e. succeeded or failed).
     void UpdateOnMoveCompleted(const std::string& ts_uuid);
 
+    // Check if all the tablets servers (excluding those specified in 'ignored_tservers')
+    // are healthy and available for replica placement.
+    Status CheckTabletServers(const rebalance::ClusterRawInfo& raw_info);
+
     // A pointer to the Rebalancer object.
     RebalancerTool* rebalancer_;
 
@@ -339,14 +343,6 @@ class RebalancerTool : public rebalance::Rebalancer {
     // Key is tserver UUID which corresponds to value.ts_uuid_from.
     typedef std::unordered_multimap<std::string, Rebalancer::ReplicaMove> MovesToSchedule;
 
-    struct TabletInfo {
-      std::string tablet_id;
-      boost::optional<int64_t> config_idx;  // For CAS-like change of Raft configs.
-    };
-
-    // Mapping tserver UUID to tablets on it.
-    typedef std::unordered_map<std::string, std::vector<TabletInfo>> IgnoredTserversInfo;
-
     // Get replica moves to move replicas from healthy ignored tservers.
     // If returns Status::OK() with replica_moves empty, there would be
     // no replica on the healthy ignored tservers.
@@ -354,11 +350,13 @@ class RebalancerTool : public rebalance::Rebalancer {
                            const rebalance::ClusterRawInfo& raw_info,
                            std::vector<Rebalancer::ReplicaMove>* replica_moves) override;
 
-    // Check the state of ignored tservers.
-    // Return Status::OK() only when all the ignored tservers are in maintenance mode.
-    Status CheckIgnoredTserversState(const rebalance::ClusterInfo& ci);
+    // Return Status::OK() if it's safe to move all replicas from the ignored to other servers
+    // and all tservers that need to empty are in maintenance mode.
+    static Status CheckIgnoredTServers(const rebalance::ClusterRawInfo& raw_info,
+                                       const rebalance::ClusterInfo& ci,
+                                       const std::unordered_set<std::string>& tservers_to_empty);
 
-    void GetMovesFromIgnoredTservers(const IgnoredTserversInfo& ignored_tservers_info,
+    void GetMovesFromIgnoredTservers(const TServersToEmptyMap& ignored_tservers_info,
                                      std::vector<Rebalancer::ReplicaMove>* replica_moves);
 
     // Random device and generator for selecting among multiple choices, when appropriate.
@@ -371,13 +369,12 @@ class RebalancerTool : public rebalance::Rebalancer {
   // 'location' means that's about cross-location rebalancing). Basically,
   // 'raw' information is just a sub-set of relevant fields of the KsckResults
   // structure filtered to contain information only for the specified location.
-  static Status KsckResultsToClusterRawInfo(
-      const boost::optional<std::string>& location,
-      const KsckResults& ksck_info,
-      rebalance::ClusterRawInfo* raw_info);
+  Status KsckResultsToClusterRawInfo(const boost::optional<std::string>& location,
+                                     const KsckResults& ksck_info,
+                                     rebalance::ClusterRawInfo* raw_info);
 
-  // Print replica count infomation on ClusterInfo::tservers_to_empty.
-  Status PrintIgnoredTserversStats(const rebalance::ClusterInfo& ci,
+  // Print replica count infomation about tservers need to empty.
+  Status PrintIgnoredTserversStats(const rebalance::ClusterRawInfo& raw_info,
                                    std::ostream& out) const;
 
   // Print information on the cross-location balance.
@@ -395,10 +392,6 @@ class RebalancerTool : public rebalance::Rebalancer {
   Status PrintPolicyViolationInfo(const rebalance::ClusterRawInfo& raw_info,
                                   std::ostream& out) const;
 
-  // Check whether it is safe to move all replicas from the ignored to other servers.
-  Status CheckIgnoredServers(const rebalance::ClusterRawInfo& raw_info,
-                             const rebalance::ClusterInfo& cluster_info);
-
   // Run rebalancing using the specified runner.
   Status RunWith(Runner* runner, RunStatus* result_status);
 
