@@ -23,8 +23,10 @@
 #include <iomanip>
 #include <iostream>
 #include <iterator>
+#include <map>
 #include <memory>
 #include <numeric>
+#include <set>
 #include <stack>
 #include <string>
 #include <unordered_map>
@@ -60,6 +62,7 @@
 #include "kudu/gutil/strings/substitute.h"
 #include "kudu/gutil/strings/util.h"
 #include "kudu/master/master.h"
+#include "kudu/master/master.pb.h"
 #include "kudu/master/master.proxy.h" // IWYU pragma: keep
 #include "kudu/rpc/messenger.h"
 #include "kudu/rpc/response_callback.h"
@@ -71,6 +74,7 @@
 #include "kudu/tools/tool.pb.h" // IWYU pragma: keep
 #include "kudu/tools/tool_action.h"
 #include "kudu/tserver/tserver.pb.h"
+#include "kudu/tserver/tserver_admin.pb.h"
 #include "kudu/tserver/tserver_admin.proxy.h"   // IWYU pragma: keep
 #include "kudu/tserver/tserver_service.proxy.h" // IWYU pragma: keep
 #include "kudu/util/async_util.h"
@@ -151,6 +155,12 @@ DEFINE_string(sasl_protocol_name,
               "servers' service principal name base (e.g. if it's \"kudu/_HOST\", then "
               "sasl_protocol_name must be \"kudu\" to be able to connect.");
 
+DEFINE_bool(row_count_only, false,
+            "Whether to only count rows instead of reading row cells: yields "
+            "an empty projection for the table");
+
+DECLARE_bool(show_values);
+
 bool ValidateTimeoutSettings() {
   if (FLAGS_timeout_ms < FLAGS_negotiation_timeout_ms) {
     LOG(ERROR) << strings::Substitute(
@@ -164,6 +174,23 @@ bool ValidateTimeoutSettings() {
 }
 GROUP_FLAG_VALIDATOR(timeout_flags, ValidateTimeoutSettings);
 
+bool ValidateSchemaProjectionFlags() {
+  if (FLAGS_row_count_only && !FLAGS_columns.empty()) {
+    LOG(ERROR) <<
+        "--row_count_only and --columns flags are conflicting: "
+        "either remove/unset --columns or remove/unset --row_count_only";
+    return false;
+  }
+  if (FLAGS_row_count_only && FLAGS_show_values) {
+    LOG(ERROR) <<
+        "--row_count_only and --show_values flags are conflicting: either "
+        "remove/unset --show_values or remove/unset --row_count_only";
+    return false;
+  }
+  return true;
+}
+GROUP_FLAG_VALIDATOR(schema_projection_flags, ValidateSchemaProjectionFlags);
+
 using kudu::client::KuduClient;
 using kudu::client::KuduClientBuilder;
 using kudu::client::internal::AsyncLeaderMasterRpc;
@@ -173,6 +200,8 @@ using kudu::consensus::ReplicateMsg;
 using kudu::log::LogEntryPB;
 using kudu::log::LogEntryReader;
 using kudu::log::ReadableLogSegment;
+using kudu::master::ConnectToMasterRequestPB;
+using kudu::master::ConnectToMasterResponsePB;
 using kudu::master::MasterServiceProxy;
 using kudu::pb_util::SecureDebugString;
 using kudu::pb_util::SecureShortDebugString;
@@ -192,12 +221,16 @@ using kudu::server::ServerClockResponsePB;
 using kudu::server::ServerStatusPB;
 using kudu::server::SetFlagRequestPB;
 using kudu::server::SetFlagResponsePB;
+using kudu::tserver::ListTabletsRequestPB;
+using kudu::tserver::ListTabletsResponsePB;
 using kudu::tserver::TabletServerAdminServiceProxy; // NOLINT
 using kudu::tserver::TabletServerServiceProxy; // NOLINT
 using kudu::tserver::WriteRequestPB;
 using std::cout;
 using std::endl;
+using std::map;
 using std::ostream;
+using std::set;
 using std::setfill;
 using std::setw;
 using std::shared_ptr;
@@ -313,7 +346,7 @@ Status PrintDecodedWriteRequestPB(const string& indent,
   return Status::OK();
 }
 
-Status PrintDecoded(const LogEntryPB& entry, const Schema& tablet_schema) {
+Status PrintDecoded(const LogEntryPB& entry, Schema* tablet_schema) {
   PrintIdOnly(entry);
 
   const string indent = "\t";
@@ -324,9 +357,16 @@ Status PrintDecoded(const LogEntryPB& entry, const Schema& tablet_schema) {
     if (replicate.op_type() == consensus::WRITE_OP) {
       RETURN_NOT_OK(PrintDecodedWriteRequestPB(
           indent,
-          tablet_schema,
+          *tablet_schema,
           replicate.write_request(),
           replicate.has_request_id() ? &replicate.request_id() : nullptr));
+    } else if (replicate.op_type() == consensus::ALTER_SCHEMA_OP) {
+      if (!replicate.has_alter_schema_request()) {
+        LOG(ERROR) << "read an ALTER_SCHEMA_OP log entry, but has no alter_schema_request";
+        return Status::RuntimeError("ALTER_SCHEMA_OP log entry has no alter_schema_request");
+      }
+      RETURN_NOT_OK(SchemaFromPB(replicate.alter_schema_request().schema(), tablet_schema));
+      cout << indent << SecureShortDebugString(replicate) << endl;
     } else {
       cout << indent << SecureShortDebugString(replicate) << endl;
     }
@@ -480,6 +520,24 @@ Status GetServerStatus(const string& address, uint16_t default_port,
   return Status::OK();
 }
 
+Status GetReplicas(TabletServerServiceProxy* proxy,
+                   vector<ListTabletsResponsePB::StatusAndSchemaPB>* replicas) {
+  ListTabletsRequestPB req;
+  req.set_need_schema_info(true);
+  ListTabletsResponsePB resp;
+  RpcController rpc;
+  rpc.set_timeout(MonoDelta::FromMilliseconds(FLAGS_timeout_ms));
+
+  RETURN_NOT_OK(proxy->ListTablets(req, &resp, &rpc));
+  if (resp.has_error()) {
+    return StatusFromPB(resp.error().status());
+  }
+
+  replicas->assign(resp.status_and_schema().begin(),
+                   resp.status_and_schema().end());
+  return Status::OK();
+}
+
 Status PrintSegment(const scoped_refptr<ReadableLogSegment>& segment) {
   PrintEntryType print_type = ParsePrintType();
   if (FLAGS_print_meta) {
@@ -505,7 +563,7 @@ Status PrintSegment(const scoped_refptr<ReadableLogSegment>& segment) {
 
         cout << "Entry:\n" << SecureDebugString(*entry);
       } else if (print_type == PRINT_DECODED) {
-        RETURN_NOT_OK(PrintDecoded(*entry, tablet_schema));
+        RETURN_NOT_OK(PrintDecoded(*entry, &tablet_schema));
       } else if (print_type == PRINT_ID) {
         PrintIdOnly(*entry);
       }
@@ -701,6 +759,61 @@ Status MasterAddressesToSet(
   return Status::OK();
 }
 
+Status VerifyMasterAddressList(const vector<string>& master_addresses) {
+  map<string, set<string>> addresses_per_master;
+  for (const auto& address : master_addresses) {
+    unique_ptr<MasterServiceProxy> proxy;
+    RETURN_NOT_OK(BuildProxy(address, master::Master::kDefaultPort, &proxy));
+
+    RpcController ctl;
+    ctl.set_timeout(MonoDelta::FromMilliseconds(FLAGS_timeout_ms));
+    ConnectToMasterRequestPB req;
+    ConnectToMasterResponsePB resp;
+    RETURN_NOT_OK(proxy->ConnectToMaster(req, &resp, &ctl));
+    const auto& resp_master_addrs = resp.master_addrs();
+    if (resp_master_addrs.size() != master_addresses.size()) {
+      const auto addresses_provided = JoinStrings(master_addresses, ",");
+      const auto addresses_cluster_config =
+          JoinMapped(resp_master_addrs,
+                     [](const HostPortPB& pb) { return Substitute("$0:$1", pb.host(), pb.port()); },
+                     ",");
+      return Status::InvalidArgument(
+          Substitute("list of master addresses provided ($0) "
+                     "does not match the actual cluster configuration ($1) ",
+                     addresses_provided,
+                     addresses_cluster_config));
+    }
+    set<string> addr_set;
+    for (const auto& hp : resp_master_addrs) {
+      addr_set.emplace(Substitute("$0:$1", hp.host(), hp.port()));
+    }
+    addresses_per_master.emplace(address, std::move(addr_set));
+  }
+
+  bool mismatch = false;
+  if (addresses_per_master.size() > 1) {
+    const auto it_0 = addresses_per_master.cbegin();
+    auto it_1 = addresses_per_master.begin();
+    ++it_1;
+    for (auto it = it_1; it != addresses_per_master.end(); ++it) {
+      if (it->second != it_0->second) {
+        mismatch = true;
+        break;
+      }
+    }
+  }
+
+  if (mismatch) {
+    string err_msg = Substitute("specified: ($0);", JoinStrings(master_addresses, ","));
+    for (const auto& e : addresses_per_master) {
+      err_msg += Substitute(" from master $0: ($1);", e.first, JoinStrings(e.second, ","));
+    }
+    return Status::ConfigurationError(Substitute("master address lists mismatch: $0", err_msg));
+  }
+
+  return Status::OK();
+}
+
 Status PrintServerStatus(const string& address, uint16_t default_port) {
   ServerStatusPB status;
   RETURN_NOT_OK(GetServerStatus(address, default_port, &status));
@@ -940,6 +1053,7 @@ Status LeaderMasterProxy::Init(const vector<string>& master_addrs,
       .default_rpc_timeout(timeout)
       .default_admin_operation_timeout(timeout)
       .connection_negotiation_timeout(connection_negotiation_timeout)
+      .sasl_protocol_name(FLAGS_sasl_protocol_name)
       .Build(&client_);
 }
 
