@@ -175,6 +175,7 @@ constexpr const char* const kPeerUUIDsArg = "peer uuids";
 constexpr const char* const kPeerUUIDsArgDesc =
     "List of peer uuids to be part of new config, separated by whitespace";
 
+
 Status GetReplicas(TabletServerServiceProxy* proxy,
                    vector<ListTabletsResponsePB::StatusAndSchemaPB>* replicas) {
   ListTabletsRequestPB req;
@@ -316,6 +317,10 @@ Status ListReplicas(const RunnerContext& context) {
     const string& state = tablet::TabletStatePB_Name(rs.state());
     cout << "Tablet id: " << rs.tablet_id() << endl;
     cout << "State: " << state << endl;
+    cout << "Last status: " << rs.last_status() << endl;
+    if (r.has_role()) {
+      cout << "Role: " << RaftPeerPB::Role_Name(r.role()) << endl;
+    }
     cout << "Table name: " << rs.table_name() << endl;
     cout << "Partition: "
          << partition_schema.PartitionDebugString(partition, schema) << endl;
@@ -323,6 +328,8 @@ Status ListReplicas(const RunnerContext& context) {
       cout << "Estimated on disk size: "
            << HumanReadableNumBytes::ToString(rs.estimated_on_disk_size()) << endl;
     }
+    const string& data_state = tablet::TabletDataState_Name(rs.tablet_data_state());
+    cout << "Data state: " << data_state << endl;
     if (rs.data_dirs_size() != 0) {
       cout << "Data dirs: " << JoinStrings(rs.data_dirs(), ", ") << endl;
     } else {
