@@ -16,9 +16,8 @@
 // under the License.
 #include "kudu/tserver/tablet_copy_client.h"
 
-#include <stdlib.h>
-
 #include <cstdint>
+#include <cstdlib>
 #include <limits>
 #include <memory>
 #include <ostream>
@@ -45,7 +44,6 @@
 #include "kudu/fs/data_dirs.h"
 #include "kudu/fs/fs_manager.h"
 #include "kudu/gutil/casts.h"
-#include "kudu/gutil/port.h"
 #include "kudu/gutil/ref_counted.h"
 #include "kudu/gutil/strings/fastmem.h"
 #include "kudu/gutil/strings/join.h"
@@ -70,11 +68,6 @@
 #include "kudu/util/test_macros.h"
 #include "kudu/util/test_util.h"
 
-using std::shared_ptr;
-using std::string;
-using std::thread;
-using std::vector;
-
 DECLARE_double(env_inject_eio);
 DECLARE_double(tablet_copy_fault_crash_during_download_block);
 DECLARE_double(tablet_copy_fault_crash_during_download_wal);
@@ -84,22 +77,30 @@ DECLARE_string(env_inject_eio_globs);
 
 METRIC_DECLARE_counter(block_manager_total_disk_sync);
 
+using kudu::consensus::ConsensusMetadataManager;
+using kudu::consensus::ConsensusStatePB;
+using kudu::consensus::GetRaftConfigLeader;
+using kudu::consensus::RaftPeerPB;
+using kudu::fs::DataDirManager;
+using kudu::tablet::TabletMetadata;
+using std::shared_ptr;
+using std::string;
+using std::thread;
+using std::tuple;
+using std::unique_ptr;
+using std::vector;
+using strings::Substitute;
+
 namespace kudu {
 namespace tserver {
 
-using consensus::ConsensusMetadataManager;
-using consensus::ConsensusStatePB;
-using consensus::GetRaftConfigLeader;
-using consensus::RaftPeerPB;
-using fs::DataDirManager;
-using std::tuple;
-using std::unique_ptr;
-using strings::Substitute;
-using tablet::TabletMetadata;
-
 class TabletCopyClientTest : public TabletCopyTest {
  public:
-  virtual void SetUp() OVERRIDE {
+  TabletCopyClientTest()
+      : rand_(SeedRandom()) {
+  }
+
+  void SetUp() override {
     NO_FATALS(TabletCopyTest::SetUp());
 
     // To be a bit more flexible in testing, create a FS layout with multiple disks.
@@ -148,14 +149,14 @@ class TabletCopyClientTest : public TabletCopyTest {
  protected:
   Status CompareFileContents(const string& path1, const string& path2);
 
-  // Injection of 'supports_live_row_count' modifiers through polymorphic characteristic.
+  // Injection of 'supports_live_row_count' modifiers.
   void GenerateTestData() override {
-    Random rand(SeedRandom());
-    NO_FATALS(tablet_replica_->tablet_metadata()->
-        set_supports_live_row_count_for_tests(rand.Next() % 2));
+    tablet_replica_->tablet_metadata()->set_supports_live_row_count_for_tests(
+        rand_.Next() % 2);
     NO_FATALS(TabletCopyTest::GenerateTestData());
   }
 
+  Random rand_;
   MetricRegistry metric_registry_;
   scoped_refptr<MetricEntity> metric_entity_;
   unique_ptr<FsManager> fs_manager_;
@@ -167,12 +168,16 @@ class TabletCopyClientTest : public TabletCopyTest {
 
 Status TabletCopyClientTest::CompareFileContents(const string& path1, const string& path2) {
   shared_ptr<RandomAccessFile> file1, file2;
-  RETURN_NOT_OK(env_util::OpenFileForRandom(fs_manager_->env(), path1, &file1));
-  RETURN_NOT_OK(env_util::OpenFileForRandom(fs_manager_->env(), path2, &file2));
+  RandomAccessFileOptions opts;
+  opts.is_sensitive = true;
+  RETURN_NOT_OK(env_util::OpenFileForRandom(opts, fs_manager_->env(), path1, &file1));
+  RETURN_NOT_OK(env_util::OpenFileForRandom(opts, fs_manager_->env(), path2, &file2));
 
   uint64_t size1, size2;
   RETURN_NOT_OK(file1->Size(&size1));
   RETURN_NOT_OK(file2->Size(&size2));
+  size1 -= file1->GetEncryptionHeaderSize();
+  size2 -= file2->GetEncryptionHeaderSize();
   if (size1 != size2) {
     return Status::Corruption("Sizes of files don't match",
                               Substitute("$0 vs $1 bytes", size1, size2));
@@ -183,8 +188,8 @@ Status TabletCopyClientTest::CompareFileContents(const string& path1, const stri
   scratch2.resize(size2);
   Slice slice1(scratch1.data(), size1);
   Slice slice2(scratch2.data(), size2);
-  RETURN_NOT_OK(file1->Read(0, slice1));
-  RETURN_NOT_OK(file2->Read(0, slice2));
+  RETURN_NOT_OK(file1->Read(file1->GetEncryptionHeaderSize(), slice1));
+  RETURN_NOT_OK(file2->Read(file2->GetEncryptionHeaderSize(), slice2));
   int result = strings::fastmemcmp_inlined(slice1.data(), slice2.data(), size1);
   if (result != 0) {
     return Status::Corruption("Files do not match");
@@ -465,7 +470,7 @@ class TabletCopyClientAbortTest : public TabletCopyClientTest,
                                   public ::testing::WithParamInterface<
                                       tuple<DownloadBlocks, DeleteTrigger>> {
  public:
-  virtual void SetUp() override {
+  void SetUp() override {
     TabletCopyClientTest::SetUp();
     ASSERT_OK(StartCopy());
   }
