@@ -32,9 +32,9 @@
 #include "kudu/common/common.pb.h"
 #include "kudu/common/partial_row.h"
 #include "kudu/common/row_operations.h"
+#include "kudu/common/row_operations.pb.h"
 #include "kudu/common/schema.h"
 #include "kudu/common/wire_protocol.h"
-#include "kudu/common/wire_protocol.pb.h"
 #include "kudu/consensus/consensus_meta_manager.h"
 #include "kudu/consensus/log.h"
 #include "kudu/consensus/log_anchor_registry.h"
@@ -189,7 +189,7 @@ class TabletCopyTest : public KuduTabletTest {
                                      prepare_pool_.get(),
                                      dns_resolver_.get()));
     ASSERT_OK(tablet_replica_->WaitUntilConsensusRunning(MonoDelta::FromSeconds(10)));
-    ASSERT_OK(tablet_replica_->consensus()->WaitUntilLeaderForTests(MonoDelta::FromSeconds(10)));
+    ASSERT_OK(tablet_replica_->consensus()->WaitUntilLeader(MonoDelta::FromSeconds(10)));
   }
 
   void TabletReplicaStateChangedCallback(const string& tablet_id, const string& reason) {
@@ -252,6 +252,7 @@ class TabletCopyTest : public KuduTabletTest {
 
     // Write the file to a temporary location.
     WritableFileOptions opts;
+    opts.is_sensitive = true;
     string path_template = GetTestPath(Substitute("test_block_$0$1.XXXXXX",
                                                   block_id.ToString(),
                                                   kTmpInfix));
@@ -260,7 +261,9 @@ class TabletCopyTest : public KuduTabletTest {
     CHECK_OK(writable_file->Append(Slice(data.data(), data.size())));
     CHECK_OK(writable_file->Close());
 
-    CHECK_OK(Env::Default()->NewSequentialFile(*path, file));
+    SequentialFileOptions seq_opts;
+    seq_opts.is_sensitive = true;
+    CHECK_OK(Env::Default()->NewSequentialFile(seq_opts, *path, file));
   }
 
   MetricRegistry metric_registry_;
@@ -334,13 +337,13 @@ TEST_F(TabletCopyTest, TestBlocksEqual) {
       buf.resize(tablet_block_size);
       Slice data2(buf.data(), tablet_block_size);
       ASSERT_OK(tablet_block->Read(0, data2));
-      uint32_t tablet_crc = crc::Crc32c(data.data(), data.size());
+      uint32_t tablet_crc = crc::Crc32c(data2.data(), data2.size());
       LOG(INFO) << "tablet block file has size of " << tablet_block_size
                 << " and CRC32C of " << tablet_crc
                 << ": " << block_id;
 
       // Compare the blocks.
-      ASSERT_EQ(tablet_block_size, session_block_size);
+      ASSERT_EQ(tablet_block_size, session_block_size - Env::Default()->GetEncryptionHeaderSize());
       ASSERT_EQ(tablet_crc, session_crc);
     }
   }
