@@ -17,18 +17,23 @@
 
 #include "kudu/util/path_util.h"
 
-// Use the POSIX version of dirname(3).
+// Use the POSIX version of basename(3)/dirname(3).
 #include <libgen.h>
 
+#if defined(__APPLE__)
+#include <sys/param.h> // for MAXPATHLEN
+#endif // defined(__APPLE__)
+
 #include <cstdlib>
 #include <cstring>
 #include <memory>
-#if defined(__APPLE__)
-#include <mutex>
-#endif // defined(__APPLE__)
 #include <ostream>
 #include <string>
 
+#if defined(__APPLE__)
+#include <cerrno>
+#endif // defined(__APPLE__)
+
 #include <glog/logging.h>
 
 #include "kudu/gutil/strings/split.h"
@@ -38,6 +43,12 @@
 #include "kudu/util/status.h"
 #include "kudu/util/subprocess.h"
 
+#if defined(__APPLE__)
+#include "kudu/gutil/port.h"
+#include "kudu/gutil/strings/substitute.h"
+#include "kudu/util/errno.h"
+#endif // defined(__APPLE__)
+
 using std::string;
 using std::unique_ptr;
 using std::vector;
@@ -91,17 +102,35 @@ vector<string> SplitPath(const string& path) {
 }
 
 string DirName(const string& path) {
-  unique_ptr<char[], FreeDeleter> path_copy(strdup(path.c_str()));
 #if defined(__APPLE__)
-  static std::mutex lock;
-  std::lock_guard<std::mutex> l(lock);
-#endif // defined(__APPLE__)
-  return ::dirname(path_copy.get());
+  char buf[MAXPATHLEN];
+  auto* ret = dirname_r(path.c_str(), buf);
+  if (PREDICT_FALSE(ret == nullptr)) {
+    int err = errno;
+    LOG(FATAL) << strings::Substitute("dirname_r() failed: $0",
+                                      ErrnoToString(err));
+  }
+  return ret;
+#else
+  unique_ptr<char[], FreeDeleter> path_copy(strdup(path.c_str()));
+  return dirname(path_copy.get());
+#endif // #if defined(__APPLE__) ... #else
 }
 
 string BaseName(const string& path) {
+#if defined(__APPLE__)
+  char buf[MAXPATHLEN];
+  auto* ret = basename_r(path.c_str(), buf);
+  if (PREDICT_FALSE(ret == nullptr)) {
+    int err = errno;
+    LOG(FATAL) << strings::Substitute("basename_r() failed: $0",
+                                      ErrnoToString(err));
+  }
+  return ret;
+#else
   unique_ptr<char[], FreeDeleter> path_copy(strdup(path.c_str()));
   return basename(path_copy.get());
+#endif // #if defined(__APPLE__) ... #else
 }
 
 Status FindExecutable(const string& binary,
