@@ -68,7 +68,7 @@ SNAPPY_VERSION=1.1.8
 SNAPPY_NAME=snappy-$SNAPPY_VERSION
 SNAPPY_SOURCE=$TP_SOURCE_DIR/$SNAPPY_NAME
 
-LZ4_VERSION=1.9.2
+LZ4_VERSION=1.9.3
 LZ4_NAME=lz4-$LZ4_VERSION
 LZ4_SOURCE=$TP_SOURCE_DIR/$LZ4_NAME
 
@@ -133,11 +133,11 @@ CURL_SOURCE=$TP_SOURCE_DIR/$CURL_NAME
 #  export NAME=crcutil-$(git rev-parse HEAD)
 #  git archive HEAD --prefix=$NAME/ -o /tmp/$NAME.tar.gz
 #  s3cmd put -P /tmp/$NAME.tar.gz s3://cloudera-thirdparty-libs/$NAME.tar.gz
-CRCUTIL_VERSION=81f8a60f67190ff1e0c9f2f6e5a07f650671a646
+CRCUTIL_VERSION=2903870057d2f1f109b245650be29e856dc8b646
 CRCUTIL_NAME=crcutil-$CRCUTIL_VERSION
 CRCUTIL_SOURCE=$TP_SOURCE_DIR/$CRCUTIL_NAME
 
-LIBUNWIND_VERSION=1.4.0
+LIBUNWIND_VERSION=1.5.0
 LIBUNWIND_NAME=libunwind-$LIBUNWIND_VERSION
 LIBUNWIND_SOURCE=$TP_SOURCE_DIR/$LIBUNWIND_NAME
 
@@ -258,3 +258,16 @@ POSTGRES_JDBC_SOURCE=$TP_SOURCE_DIR/$POSTGRES_JDBC_NAME
 RANGER_VERSION=2.1.0
 RANGER_NAME=ranger-$RANGER_VERSION-admin
 RANGER_SOURCE=$TP_SOURCE_DIR/$RANGER_NAME
+
+OATPP_VERSION=1.2.5
+OATPP_NAME=oatpp-$OATPP_VERSION
+OATPP_SOURCE=$TP_SOURCE_DIR/$OATPP_NAME
+
+OATPP_SWAGGER_VERSION=1.2.5
+OATPP_SWAGGER_NAME=oatpp-swagger-$OATPP_SWAGGER_VERSION
+OATPP_SWAGGER_SOURCE=$TP_SOURCE_DIR/$OATPP_SWAGGER_NAME
+
+JWT_CPP_VERSION=3bd600762a70faccc7ec1c2dacb999cba6c6ef5e
+JWT_CPP=jwt-cpp
+JWT_CPP_NAME=$JWT_CPP-$JWT_CPP_VERSION
+JWT_CPP_SOURCE=$TP_SOURCE_DIR/$JWT_CPP_NAME
